      SUBROUTINE F01AEF(N,A,IA,B,IB,DL,IFAIL)
C     MARK 2 RELEASE. NAG COPYRIGHT 1972
C     MARK 3 REVISED.
C     MARK 4.5 REVISED
C     MARK 11 REVISED. VECTORISATION (JAN 1984).
C     MARK 11.5(F77) REVISED. (SEPT 1985.)
C     MARK 12 REVISED. EXTENDED BLAS (JUNE 1986)
C
C     REDUC1
C     REDUCTION OF THE GENERAL SYMMETRIC EIGENVALUE PROBLEM A * X =
C     LAMBDA * B * X, WITH SYMMETRIC MATRIX A AND SYMMETRIC
C     POSITIVE DEFINITE MATRIX B, TO THE EQUIVALENT STANDARD
C     PROBLEM P * Z = LAMBDA * Z. THE UPPER TRIANGLE, INCLUDING
C     DIAGONAL ELEMENTS, OF A AND B ARE GIVEN IN THE ARRAYS A(N,N)
C     AND B(N,N). L(B = L * LT) IS FORMED IN THE
C     REMAINING STRICTLY LOWER TRIANGLE OF THE ARRAY B WITH ITS
C     DIAGONAL ELEMENTS IN THE ARRAY DL(N) , AND THE LOWER
C     TRIANGLE OF THE SYMMETRIC MATRIX P(P = INV(L) * A * INV( LT))
C     IS FORMED IN THE LOWER TRIANGLE OF THE ARRAY A, INCLUDING THE
C     DIAGONAL ELEMENTS. HENCE THE DIAGONAL ELEMENTS OF A ARE LOST.
C     THE SUBROUTINE WILL FAIL IF B, PERHAPS ON ACCOUNT OF ROUNDING
C     ERRORS, IS NOT POSITIVE DEFINITE - SETS IFAIL = 0 IF
C     SUCCESSFUL ELSE IFAIL = 1.
C     1ST DECEMBER 1971
C
C     .. Parameters ..
      CHARACTER*6       SRNAME
      PARAMETER         (SRNAME='F01AEF')
C     .. Scalar Arguments ..
      INTEGER           IA, IB, IFAIL, N
C     .. Array Arguments ..
      DOUBLE PRECISION  A(IA,N), B(IB,N), DL(N)
C     .. Local Scalars ..
      DOUBLE PRECISION  X, Y
      INTEGER           I, IB1, J
C     .. Local Arrays ..
      CHARACTER*1       P01REC(1)
C     .. External Functions ..
      INTEGER           P01ABF
      EXTERNAL          P01ABF
C     .. External Subroutines ..
      EXTERNAL          DGEMV, DSWAP, DTRSV
C     .. Intrinsic Functions ..
      INTRINSIC         SQRT
C     .. Executable Statements ..
C     COPY UPPER TRIANGLES OF A AND B INTO LOWER TRIANGLES
C     AND DIAGONAL OF B INTO DL
      DO 40 J = 1, N
         DO 20 I = 1, J
            A(J,I) = A(I,J)
            B(J,I) = B(I,J)
   20    CONTINUE
         DL(J) = B(J,J)
   40 CONTINUE
      DO 80 I = 1, N
         X = B(I,I)
         IF (X.LE.0.0D0) GO TO 160
         X = SQRT(X)
         B(I,I) = X
         IF (I.GE.N) GO TO 80
         DO 60 J = I + 1, N
            B(J,I) = B(J,I)/X
   60    CONTINUE
         CALL DGEMV('N',N-I,I,-1.0D0,B(I+1,1),IB,B(I+1,1),IB,1.0D0,
     *              B(I+1,I+1),1)
   80 CONTINUE
C     L HAS BEEN FORMED IN ARRAY B
      DO 120 I = 1, N
         Y = B(I,I)
         CALL DGEMV('N',N-I+1,I-1,-1.0D0,A(I,1),IA,B(I,1),IB,1.0D0,A(I,
     *              I),1)
         DO 100 J = I, N
            A(J,I) = A(J,I)/Y
  100    CONTINUE
  120 CONTINUE
C     THE TRANSPOSE OF THE UPPER TRIANGLE OF
C     INV(L) * A HAS BEEN FORMED IN THE LOWER
C     TRIANGLE OF ARRAY A
      IB1 = IB
      DO 140 J = 1, N
         CALL DGEMV('N',N-J+1,J-1,-1.0D0,B(J,1),IB,A(J,1),IA,1.0D0,A(J,
     *              J),1)
         IF (J.EQ.N) IB1 = 1
         CALL DTRSV('L','N','N',N-J+1,B(J,J),IB1,A(J,J),1)
  140 CONTINUE
      CALL DSWAP(N,DL,1,B,IB+1)
      IFAIL = 0
      RETURN
  160 IFAIL = P01ABF(IFAIL,1,SRNAME,0,P01REC)
      DO 180 J = 1, N
         B(J,J) = DL(J)
  180 CONTINUE
      RETURN
      END
      SUBROUTINE F01AFF(N,IM1,IM2,B,IB,DL,Z,IZ)
C     MARK 2 RELEASE. NAG COPYRIGHT 1972
C     MARK 4.5 REVISED
C     MARK 11 REVISED. VECTORISATION (JAN 1984).
C     MARK 11.5(F77) REVISED. (SEPT 1985.)
C     MARK 12 REVISED. EXTENDED BLAS (JUNE 1986)
C
C     REBAKA
C     THIS SUBROUTINE PERFORMS, ON THE MATRIX OF EIGENVECTORS, Z,
C     STORED
C     IN COLUMNS IM1 TO IM2 OF THE ARRAY Z(N,N), A BACKWARD
C     SUBSTITUTION
C     LT* X = Z, OVER- WRITING X ON Z. THE DIAGONAL ELEMENTS OF L
C     MUST BE STORED IN THE ARRAY DL(N), AND THE REMAINING
C     TRIANGLE IN THE STRICTLY LOWER TRIANGLE OF THE ARRAY B(N,N).
C     THE SUBROUTINES F01AEF AND F01BDF LEAVE L IN THIS
C     DESIRED FORM. IF X DENOTES ANY COLUMN OF THE RESULTANT MATRIX
C     X, THEN X SATISFIES XT * B * X = ZT * Z, WHERE B = L * LT.
C     1ST AUGUST 1971
C
C     .. Scalar Arguments ..
      INTEGER           IB, IM1, IM2, IZ, N
C     .. Array Arguments ..
      DOUBLE PRECISION  B(IB,N), DL(N), Z(IZ,IM2)
C     .. Local Scalars ..
      INTEGER           J
C     .. External Subroutines ..
      EXTERNAL          DSWAP, DTRSV
C     .. Executable Statements ..
      CALL DSWAP(N,DL,1,B,IB+1)
      DO 20 J = IM1, IM2
         CALL DTRSV('L','T','N',N,B,IB,Z(1,J),1)
   20 CONTINUE
      CALL DSWAP(N,DL,1,B,IB+1)
      RETURN
      END
      SUBROUTINE F01AJF(N,ATOL,A,IA,D,E,Z,IZ)
C     MARK 14 RE-ISSUE.  NAG COPYRIGHT 1989.
C
C     TRED2
C     THIS SUBROUTINE REDUCES THE GIVEN LOWER TRIANGLE OF A
C     SYMMETRIC MATRIX, A, STORED IN THE ARRAY A(N,N), TO
C     TRIDIAGONAL FORM USING HOUSEHOLDERS REDUCTION. THE DIAGONAL
C     OF THE RESULT IS STORED IN THE ARRAY D(N) AND THE
C     SUB-DIAGONAL IN THE LAST N - 1 STORES OF THE ARRAY E(N)
C     (WITH THE ADDITIONAL ELEMENT E(1) = 0). THE TRANSFORMATION
C     MATRICES ARE ACCUMULATED IN THE ARRAY Z(N,N). THE ARRAY
C     A IS LEFT UNALTERED UNLESS THE ACTUAL PARAMETERS
C     CORRESPONDING TO A AND Z ARE IDENTICAL.
C     1ST AUGUST 1971
C
C     REVISED BY VINCE FERNANDO AT MARK 14 TO INTRODUCE SCALING INTO
C     THE GENERATION OF HOUSEHOLDER MATRICES AS PROPOSED BY
C     G.W. STEWART, INTRODUCTION TO MATRIX COMPUTATIONS, CHAPTER 7.
C     ATOL IS NOW A DUMMY PARAMETER.
C
C     .. Scalar Arguments ..
      DOUBLE PRECISION  ATOL
      INTEGER           IA, IZ, N
C     .. Array Arguments ..
      DOUBLE PRECISION  A(IA,N), D(N), E(N), Z(IZ,N)
C     .. Local Scalars ..
      DOUBLE PRECISION  F, G, H, HH, SCALE
      INTEGER           I, II, J, K, L
C     .. External Functions ..
      DOUBLE PRECISION  DDOT
      INTEGER           IDAMAX
      EXTERNAL          DDOT, IDAMAX
C     .. External Subroutines ..
      EXTERNAL          DGEMV, DGER, DSCAL, DSYMV, DSYR2
C     .. Intrinsic Functions ..
      INTRINSIC         ABS, SQRT
C     .. Executable Statements ..
      DO 40 I = 1, N
         DO 20 J = I, N
            Z(J,I) = A(J,I)
   20    CONTINUE
         D(I) = A(N,I)
   40 CONTINUE
      IF (N.EQ.1) GO TO 340
      DO 240 II = 2, N
         I = N - II + 2
         L = I - 1
         IF (L.EQ.1) GO TO 60
C        FIND THE ELEMENT OF LARGEST ABSOLUTE VALUE IN D
         K = IDAMAX(L,D,1)
         SCALE = ABS(D(K))
C        IF D IS A NULL VECTOR THEN SKIP THE TRANSFORMATION
         IF (SCALE.GT.0.0D0) GO TO 120
   60    E(I) = D(L)
         H = 0.0D0
         DO 80 J = 1, L
            Z(J,I) = 0.0D0
   80    CONTINUE
         DO 100 J = 1, L
            Z(I,J) = 0.0D0
            D(J) = Z(I-1,J)
  100    CONTINUE
         GO TO 220
  120    CALL DSCAL(L,1.0D0/SCALE,D,1)
         H = DDOT(L,D,1,D,1)
         F = D(I-1)
         G = SQRT(H)
         IF (F.GE.0.0D0) G = -G
         E(I) = G*SCALE
         H = H - F*G
         D(I-1) = F - G
C        COPY U
         DO 140 J = 1, L
            Z(J,I) = D(J)
  140    CONTINUE
C        FORM A*U
         CALL DSYMV('L',L,1.0D0/H,Z,IZ,D,1,0.0D0,E,1)
C        FORM P
         F = 0.0D0
         DO 160 J = 1, L
            F = F + E(J)*D(J)
  160    CONTINUE
C        FORM K
         HH = F/(H+H)
C        FORM Q
         DO 180 J = 1, L
            E(J) = E(J) - HH*D(J)
  180    CONTINUE
C        FORM REDUCED A
         CALL DSYR2('L',L,-1.0D0,D,1,E,1,Z,IZ)
         DO 200 J = 1, L
            D(J) = Z(L,J)
            Z(I,J) = 0.0D0
  200    CONTINUE
  220    D(I) = H
  240 CONTINUE
C     ACCUMULATION OF TRANSFORMATION MATRICES
      DO 300 I = 2, N
         L = I - 1
         Z(N,L) = Z(L,L)
         Z(L,L) = 1.0D0
         H = D(I)
         IF (H.EQ.0.0D0) GO TO 260
         CALL DGEMV('T',L,L,1.0D0/H,Z,IZ,Z(1,I),1,0.0D0,D,1)
         CALL DGER(L,L,-1.0D0,Z(1,I),1,D,1,Z,IZ)
  260    DO 280 J = 1, L
            Z(J,I) = 0.0D0
  280    CONTINUE
  300 CONTINUE
      DO 320 I = 1, N
         D(I) = Z(N,I)
         Z(N,I) = 0.0D0
  320 CONTINUE
  340 Z(N,N) = 1.0D0
      E(1) = 0.0D0
      RETURN
      END
      SUBROUTINE F02ABF(A,IA,N,R,V,IV,E,IFAIL)
C     MARK 2 RELEASE. NAG COPYRIGHT 1972
C     MARK 3 REVISED.
C     MARK 4.5 REVISED
C     MARK 11.5(F77) REVISED. (SEPT 1985.)
C     MARK 13 REVISED. USE OF MARK 12 X02 FUNCTIONS (APR 1988).
C     MARK 14 REVISED. IER-735 (DEC 1989).
C
C     EIGENVALUES AND EIGENVECTORS OF A REAL SYMMETRIX MATRIX
C     1ST AUGUST 1971
C
C     .. Parameters ..
      CHARACTER*6       SRNAME
      PARAMETER         (SRNAME='F02ABF')
C     .. Scalar Arguments ..
      INTEGER           IA, IFAIL, IV, N
C     .. Array Arguments ..
      DOUBLE PRECISION  A(IA,N), E(N), R(N), V(IV,N)
C     .. Local Scalars ..
      DOUBLE PRECISION  XXXX
      INTEGER           ISAVE
C     .. Local Arrays ..
      CHARACTER*1       P01REC(1)
C     .. External Functions ..
      DOUBLE PRECISION  X02AJF
      INTEGER           P01ABF
      EXTERNAL          X02AJF, P01ABF
C     .. External Subroutines ..
      EXTERNAL          F01AJF, F02AMF
C     .. Executable Statements ..
      ISAVE = IFAIL
      IFAIL = 1
      CALL F01AJF(N,XXXX,A,IA,R,E,V,IV)
      CALL F02AMF(N,X02AJF(),R,E,V,IV,IFAIL)
      IF (IFAIL.NE.0) IFAIL = P01ABF(ISAVE,IFAIL,SRNAME,0,P01REC)
      RETURN
      END
      SUBROUTINE F02AEF(A,IA,B,IB,N,R,V,IV,DL,E,IFAIL)
C     MARK 2 RELEASE. NAG COPYRIGHT 1972
C     MARK 3 REVISED.
C     MARK 4.5 REVISED
C     MARK 11.5(F77) REVISED. (SEPT 1985.)
C     MARK 13 REVISED. USE OF MARK 12 X02 FUNCTIONS (APR 1988).
C     MARK 14 REVISED. IER-737 (DEC 1989).
C
C     EIGENVALUES AND EIGENVECTORS OF A-LAMBDA*B
C     1ST DECEMBER 1971
C
C     .. Parameters ..
      CHARACTER*6       SRNAME
      PARAMETER         (SRNAME='F02AEF')
C     .. Scalar Arguments ..
      INTEGER           IA, IB, IFAIL, IV, N
C     .. Array Arguments ..
      DOUBLE PRECISION  A(IA,N), B(IB,N), DL(N), E(N), R(N), V(IV,N)
C     .. Local Scalars ..
      DOUBLE PRECISION  XXXX
      INTEGER           ISAVE
C     .. Local Arrays ..
      CHARACTER*1       P01REC(1)
C     .. External Functions ..
      DOUBLE PRECISION  X02AJF
      INTEGER           P01ABF
      EXTERNAL          X02AJF, P01ABF
C     .. External Subroutines ..
      EXTERNAL          F01AEF, F01AFF, F01AJF, F02AMF
C     .. Executable Statements ..
      ISAVE = IFAIL
      IFAIL = 1
      CALL F01AEF(N,A,IA,B,IB,DL,IFAIL)
      IF (IFAIL.EQ.0) GO TO 20
      IFAIL = P01ABF(ISAVE,IFAIL,SRNAME,0,P01REC)
      RETURN
   20 CONTINUE
      CALL F01AJF(N,XXXX,A,IA,R,E,V,IV)
      IFAIL = 1
      CALL F02AMF(N,X02AJF(),R,E,V,IV,IFAIL)
      IF (IFAIL.EQ.0) GO TO 40
      IFAIL = P01ABF(ISAVE,2,SRNAME,0,P01REC)
      RETURN
   40 CALL F01AFF(N,1,N,B,IB,DL,V,IV)
      RETURN
      END
      SUBROUTINE F02AMF(N,EPS,D,E,Z,IZ,IFAIL)
C     MARK 14 RE-ISSUE.  NAG COPYRIGHT 1989.
C
C     TQL2
C     THIS SUBROUTINE FINDS THE EIGENVALUES AND EIGENVECTORS OF A
C     TRIDIAGONAL MATRIX, T, GIVEN WITH ITS DIAGONAL ELEMENTS IN
C     THE ARRAY D(N) AND ITS SUB-DIAGONAL ELEMENTS IN THE LAST N
C     - 1 STORES OF THE ARRAY E(N), USING QL TRANSFORMATIONS. THE
C     EIGENVALUES ARE OVERWRITTEN ON THE DIAGONAL ELEMENTS IN THE
C     ARRAY D IN ASCENDING ORDER. THE EIGENVECTORS ARE FORMED IN
C     THE ARRAY Z(N,N), OVERWRITING THE ACCUMULATED
C     TRANSFORMATIONS AS SUPPLIED BY THE SUBROUTINE F01AJF. THE
C     SUBROUTINE WILL FAIL IF ALL EIGENVALUES TAKE MORE THAN 30*N
C     ITERATIONS.
C     1ST APRIL 1972
C
C     .. Parameters ..
      INTEGER           VLEN
      PARAMETER         (VLEN=128)
      CHARACTER*6       SRNAME
      PARAMETER         (SRNAME='F02AMF')
C     .. Scalar Arguments ..
      DOUBLE PRECISION  EPS
      INTEGER           IFAIL, IZ, N
C     .. Array Arguments ..
      DOUBLE PRECISION  D(N), E(N), Z(IZ,N)
C     .. Local Scalars ..
      DOUBLE PRECISION  B, C, F, G, H, P, R, S
      INTEGER           I, I1, IPOS, ISAVE, ISEG, J, K, L, M
C     .. Local Arrays ..
      DOUBLE PRECISION  CC(VLEN), SS(VLEN)
      CHARACTER*1       P01REC(1)
C     .. External Functions ..
      INTEGER           P01ABF
      EXTERNAL          P01ABF
C     .. External Subroutines ..
      EXTERNAL          F06QXF
C     .. Intrinsic Functions ..
      INTRINSIC         ABS, MAX, SQRT
C     .. Executable Statements ..
      ISAVE = IFAIL
      IF (N.EQ.1) GO TO 40
      DO 20 I = 2, N
         E(I-1) = E(I)
   20 CONTINUE
   40 E(N) = 0.0D0
      B = 0.0D0
      F = 0.0D0
      J = 30*N
      DO 300 L = 1, N
         H = EPS*(ABS(D(L))+ABS(E(L)))
         IF (B.LT.H) B = H
C        LOOK FOR SMALL SUB-DIAG ELEMENT
         DO 60 M = L, N
            IF (ABS(E(M)).LE.B) GO TO 80
   60    CONTINUE
   80    IF (M.EQ.L) GO TO 280
  100    IF (J.LE.0) GO TO 400
         J = J - 1
C        FORM SHIFT
         G = D(L)
         H = D(L+1) - G
         IF (ABS(H).GE.ABS(E(L))) GO TO 120
         P = H*0.5D0/E(L)
         R = SQRT(P*P+1.0D0)
         H = P + R
         IF (P.LT.0.0D0) H = P - R
         D(L) = E(L)/H
         GO TO 140
  120    P = 2.0D0*E(L)/H
         R = SQRT(P*P+1.0D0)
         D(L) = E(L)*P/(1.0D0+R)
  140    H = G - D(L)
         I1 = L + 1
         IF (I1.GT.N) GO TO 180
         DO 160 I = I1, N
            D(I) = D(I) - H
  160    CONTINUE
  180    F = F + H
C        QL TRANSFORMATION
         P = D(M)
         C = 1.0D0
         S = 0.0D0
         DO 260 K = M - 1, L, -VLEN
            ISEG = MAX(K-VLEN+1,L)
            DO 240 I = K, ISEG, -1
               G = C*E(I)
               H = C*P
               IF (ABS(P).LT.ABS(E(I))) GO TO 200
               C = E(I)/P
               R = SQRT(C*C+1.0D0)
               E(I+1) = S*P*R
               S = C/R
               C = 1.0D0/R
               GO TO 220
  200          C = P/E(I)
               R = SQRT(C*C+1.0D0)
               E(I+1) = S*E(I)*R
               S = 1.0D0/R
               C = C/R
  220          P = C*D(I) - S*G
               D(I+1) = H + S*(C*G+S*D(I))
C           STORE ROTATIONS
               CC(VLEN-K+I) = C
               SS(VLEN-K+I) = -S
  240       CONTINUE
C        UPDATE VECTORS
            IPOS = VLEN - K + ISEG
            CALL F06QXF('Right','Variable','Backward',N,K-ISEG+2,1,
     *                  K-ISEG+2,CC(IPOS),SS(IPOS),Z(1,ISEG),IZ)
  260    CONTINUE
         E(L) = S*P
         D(L) = C*P
         IF (ABS(E(L)).GT.B) GO TO 100
  280    D(L) = D(L) + F
  300 CONTINUE
C     ORDER EIGENVALUES AND EIGENVECTORS
      DO 380 I = 1, N
         K = I
         P = D(I)
         I1 = I + 1
         IF (I1.GT.N) GO TO 340
         DO 320 J = I1, N
            IF (D(J).GE.P) GO TO 320
            K = J
            P = D(J)
  320    CONTINUE
  340    IF (K.EQ.I) GO TO 380
         D(K) = D(I)
         D(I) = P
         DO 360 J = 1, N
            P = Z(J,I)
            Z(J,I) = Z(J,K)
            Z(J,K) = P
  360    CONTINUE
  380 CONTINUE
      IFAIL = 0
      RETURN
  400 IFAIL = P01ABF(ISAVE,1,SRNAME,0,P01REC)
      RETURN
      END
      SUBROUTINE F06AAZ ( SRNAME, INFO )
C     MARK 12 RELEASE. NAG COPYRIGHT 1986.
C     .. Scalar Arguments ..
      INTEGER            INFO
      CHARACTER*13       SRNAME
C     ..
C
C  Purpose
C  =======
C
C  F06AAZ  is an error handler for the Level 2 BLAS routines.
C
C  It is called by the Level 2 BLAS routines if an input parameter is
C  invalid.
C
C  Parameters
C  ==========
C
C  SRNAME - CHARACTER*13.
C           On entry, SRNAME specifies the name of the routine which
C           called F06AAZ.
C
C  INFO   - INTEGER.
C           On entry, INFO specifies the position of the invalid
C           parameter in the parameter-list of the calling routine.
C
C
C  Auxiliary routine for Level 2 Blas.
C
C  Written on 20-July-1986.
C
C     .. Local Scalars ..
      INTEGER            IFAIL
      CHARACTER*80       REC (1)
C     .. External Functions ..
      INTEGER            P01ABF
      EXTERNAL           P01ABF
C     ..
C     .. Executable Statements ..
      WRITE (REC (1),99999) SRNAME, INFO
      IFAIL = 0
      IFAIL = P01ABF (IFAIL, -1, SRNAME(1:6), 1, REC)
C
      RETURN
C
99999 FORMAT ( ' ** On entry to ', A13, ' parameter number ', I2,
     $         ' had an illegal value' )
C
C     End of F06AAZ.
C
      END
      DOUBLE PRECISION FUNCTION F06EAF( N, X, INCX, Y, INCY )
C     MARK 12 RELEASE. NAG COPYRIGHT 1986.
C     .. Entry Points ..
      DOUBLE PRECISION          DDOT
      ENTRY                     DDOT  ( N, X, INCX, Y, INCY )
C     .. Scalar Arguments ..
      INTEGER                           INCX, INCY, N
C     .. Array Arguments ..
      DOUBLE PRECISION                  X( * ), Y( * )
C     ..
C
C  F06EAF returns the value
C
C     F06EAF = x'y
C
C
C  Nag Fortran 77 version of the Blas routine DDOT.
C  Nag Fortran 77 O( n ) basic linear algebra routine.
C
C  -- Written on 21-September-1982.
C     Sven Hammarling, Nag Central Office.
C
C
C     .. Parameters ..
      DOUBLE PRECISION      ZERO
      PARAMETER           ( ZERO = 0.0D+0 )
C     .. Local Scalars ..
      DOUBLE PRECISION      SUM
      INTEGER               I, IX, IY
C     ..
C     .. Executable Statements ..
      SUM = ZERO
      IF( N.GT.0 )THEN
         IF( ( INCX.EQ.INCY ).AND.( INCX.GT.0 ) )THEN
            DO 10, IX = 1, 1 + ( N - 1 )*INCX, INCX
               SUM = SUM + X( IX )*Y( IX )
   10       CONTINUE
         ELSE
            IF( INCY.GE.0 )THEN
               IY = 1
            ELSE
               IY = 1 - ( N - 1 )*INCY
            END IF
            IF( INCX.GT.0 )THEN
               DO 20, IX = 1, 1 + ( N - 1 )*INCX, INCX
                  SUM = SUM + X( IX )*Y( IY )
                  IY  = IY  + INCY
   20          CONTINUE
            ELSE
               IX = 1 - ( N - 1 )*INCX
               DO 30, I = 1, N
                  SUM = SUM + X( IX )*Y( IY )
                  IX  = IX  + INCX
                  IY  = IY  + INCY
   30          CONTINUE
            END IF
         END IF
      END IF
C
      F06EAF = SUM
      RETURN
C
C     End of F06EAF. ( DDOT )
C
      END
      SUBROUTINE F06EDF( N, ALPHA, X, INCX )
C     MARK 12 RELEASE. NAG COPYRIGHT 1986.
C     .. Entry Points ..
      ENTRY      DSCAL ( N, ALPHA, X, INCX )
C     .. Scalar Arguments ..
      DOUBLE PRECISION   ALPHA
      INTEGER            INCX, N
C     .. Array Arguments ..
      DOUBLE PRECISION   X( * )
C     ..
C
C  F06EDF performs the operation
C
C     x := alpha*x
C
C
C  Nag Fortran 77 version of the Blas routine DSCAL.
C  Nag Fortran 77 O( n ) basic linear algebra routine.
C
C  -- Written on 26-November-1982.
C     Sven Hammarling, Nag Central Office.
C
C
C     .. Parameters ..
      DOUBLE PRECISION   ONE         , ZERO
      PARAMETER        ( ONE = 1.0D+0, ZERO = 0.0D+0 )
C     .. Local Scalars ..
      INTEGER            IX
C     ..
C     .. Executable Statements ..
      IF( N.GT.0 )THEN
         IF( ALPHA.EQ.ZERO )THEN
            DO 10, IX = 1, 1 + ( N - 1 )*INCX, INCX
               X( IX ) = ZERO
   10       CONTINUE
         ELSE IF( ALPHA.EQ.( -ONE ) )THEN
            DO 20, IX = 1, 1 + ( N - 1 )*INCX, INCX
               X( IX ) = -X( IX )
   20       CONTINUE
         ELSE IF( ALPHA.NE.ONE )THEN
            DO 30, IX = 1, 1 + ( N - 1 )*INCX, INCX
               X( IX ) = ALPHA*X( IX )
   30       CONTINUE
         END IF
      END IF
C
      RETURN
C
C     End of F06EDF. ( DSCAL )
C
      END
      SUBROUTINE F06EGF( N, X, INCX, Y, INCY )
C     MARK 12 RELEASE. NAG COPYRIGHT 1986.
C     .. Entry Points ..
      ENTRY      DSWAP ( N, X, INCX, Y, INCY )
C     .. Scalar Arguments ..
      INTEGER            INCX, INCY, N
C     .. Array Arguments ..
      DOUBLE PRECISION   X( * ), Y( * )
C     ..
C
C  F06EGF performs the operations
C
C     temp := x,   x := y,   y := temp.
C
C
C  Nag Fortran 77 version of the Blas routine DSWAP.
C  Nag Fortran 77 O( n ) basic linear algebra routine.
C
C  -- Written on 26-November-1982.
C     Sven Hammarling, Nag Central Office.
C
C
C     .. Local Scalars ..
      DOUBLE PRECISION   TEMP
      INTEGER            I, IX, IY
C     ..
C     .. Executable Statements ..
      IF( N.GT.0 )THEN
         IF( ( INCX.EQ.INCY ).AND.( INCY.GT.0 ) )THEN
            DO 10, IY = 1, 1 + ( N - 1 )*INCY, INCY
               TEMP    = X( IY )
               X( IY ) = Y( IY )
               Y( IY ) = TEMP
   10       CONTINUE
         ELSE
            IF( INCX.GE.0 )THEN
               IX = 1
            ELSE
               IX = 1 - ( N - 1 )*INCX
            END IF
            IF( INCY.GT.0 )THEN
               DO 20, IY = 1, 1 + ( N - 1 )*INCY, INCY
                  TEMP    = X( IX )
                  X( IX ) = Y( IY )
                  Y( IY ) = TEMP
                  IX      = IX      + INCX
   20          CONTINUE
            ELSE
               IY = 1 - ( N - 1 )*INCY
               DO 30, I = 1, N
                  TEMP    = X( IX )
                  X( IX ) = Y( IY )
                  Y( IY ) = TEMP
                  IY      = IY      + INCY
                  IX      = IX      + INCX
   30          CONTINUE
            END IF
         END IF
      END IF
C
      RETURN
C
C     End of F06EGF. ( DSWAP )
C
      END
      INTEGER FUNCTION F06JLF( N, X, INCX )
C     MARK 12 RELEASE. NAG COPYRIGHT 1986.
C     .. Entry Points ..
      INTEGER          IDAMAX
      ENTRY            IDAMAX( N, X, INCX )
C     .. Scalar Arguments ..
      INTEGER                  INCX, N
C     .. Array Arguments ..
      DOUBLE PRECISION         X( * )
C     ..
C
C  F06JLF returns the smallest value of i such that
C
C     abs( x( i ) ) = max( abs( x( j ) ) )
C                      j
C
C
C  Nag Fortran 77 version of the Blas routine IDAMAX.
C  Nag Fortran 77 O( n ) basic linear algebra routine.
C
C  -- Written on 31-May-1983.
C     Sven Hammarling, Nag Central Office.
C
C
C     .. Local Scalars ..
      DOUBLE PRECISION         XMAX
      INTEGER                  I, IMAX, IX
C     .. Intrinsic Functions ..
      INTRINSIC                ABS
C     ..
C     .. Executable Statements ..
      IF( N.GT.0 )THEN
         IMAX = 1
         IF( N.GT.1 )THEN
            XMAX = ABS( X( 1 ) )
            IX   = 1
            DO 10, I = 2, N
               IX = IX + INCX
               IF( XMAX.LT.ABS( X( IX ) ) )THEN
                  XMAX = ABS( X( IX ) )
                  IMAX = I
               END IF
   10       CONTINUE
         END IF
      ELSE
         IMAX = 0
      END IF
C
      F06JLF = IMAX
      RETURN
C
C     End of F06JLF. ( IDAMAX )
C
      END
      SUBROUTINE F06PAF( TRANS, M, N, ALPHA, A, LDA, X, INCX,
     $                   BETA, Y, INCY )
C     MARK 12 RELEASE. NAG COPYRIGHT 1986.
C     AXP4 VERSION FOR VECTOR MACHINES
C     .. Entry Points ..
      ENTRY      DGEMV ( TRANS, M, N, ALPHA, A, LDA, X, INCX,
     $                   BETA, Y, INCY )
C     .. Scalar Arguments ..
      DOUBLE PRECISION   ALPHA, BETA
      INTEGER            INCX, INCY, LDA, M, N
      CHARACTER*1        TRANS
C     .. Array Arguments ..
      DOUBLE PRECISION   A( LDA, * ), X( * ), Y( * )
C     ..
C
C  Purpose
C  =======
C
C  DGEMV  performs one of the matrix-vector operations
C
C     y := alpha*A*x + beta*y,   or   y := alpha*A'*x + beta*y,
C
C  where alpha and beta are scalars, x and y are vectors and A is an
C  m by n matrix.
C
C  Parameters
C  ==========
C
C  TRANS  - CHARACTER*1.
C           On entry, TRANS specifies the operation to be performed as
C           follows:
C
C              TRANS = 'N' or 'n'   y := alpha*A*x + beta*y.
C
C              TRANS = 'T' or 't'   y := alpha*A'*x + beta*y.
C
C              TRANS = 'C' or 'c'   y := alpha*A'*x + beta*y.
C
C           Unchanged on exit.
C
C  M      - INTEGER.
C           On entry, M specifies the number of rows of the matrix A.
C           M must be at least zero.
C           Unchanged on exit.
C
C  N      - INTEGER.
C           On entry, N specifies the number of columns of the matrix A.
C           N must be at least zero.
C           Unchanged on exit.
C
C  ALPHA  - DOUBLE PRECISION.
C           On entry, ALPHA specifies the scalar alpha.
C           Unchanged on exit.
C
C  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
C           Before entry, the leading m by n part of the array A must
C           contain the matrix of coefficients.
C           Unchanged on exit.
C
C  LDA    - INTEGER.
C           On entry, LDA specifies the first dimension of A as declared
C           in the calling (sub) program. LDA must be at least
C           max( 1, m ).
C           Unchanged on exit.
C
C  X      - DOUBLE PRECISION array of DIMENSION at least
C           ( 1 + ( n - 1 )*abs( INCX ) ) when TRANS = 'N' or 'n'
C           and at least
C           ( 1 + ( m - 1 )*abs( INCX ) ) otherwise.
C           Before entry, the incremented array X must contain the
C           vector x.
C           Unchanged on exit.
C
C  INCX   - INTEGER.
C           On entry, INCX specifies the increment for the elements of
C           X. INCX must not be zero.
C           Unchanged on exit.
C
C  BETA   - DOUBLE PRECISION.
C           On entry, BETA specifies the scalar beta. When BETA is
C           supplied as zero then Y need not be set on input.
C           Unchanged on exit.
C
C  Y      - DOUBLE PRECISION array of DIMENSION at least
C           ( 1 + ( m - 1 )*abs( INCY ) ) when TRANS = 'N' or 'n'
C           and at least
C           ( 1 + ( n - 1 )*abs( INCY ) ) otherwise.
C           Before entry with BETA non-zero, the incremented array Y
C           must contain the vector y. On exit, Y is overwritten by the
C           updated vector y.
C
C  INCY   - INTEGER.
C           On entry, INCY specifies the increment for the elements of
C           Y. INCY must not be zero.
C           Unchanged on exit.
C
C
C  Level 2 Blas routine.
C
C  -- Written on 22-October-1986.
C     Jack Dongarra, Argonne National Lab.
C     Jeremy Du Croz, Nag Central Office.
C     Sven Hammarling, Nag Central Office.
C     Richard Hanson, Sandia National Labs.
C  -- DO-loops unrolled on 20-November-1986.
C     Peter Mayes, Nag Central Office.
C
C
C     .. Parameters ..
      DOUBLE PRECISION   ONE         , ZERO
      PARAMETER        ( ONE = 1.0D+0, ZERO = 0.0D+0 )
C     .. Local Scalars ..
      DOUBLE PRECISION   TEMP, TEMP1, TEMP2, TEMP3, TEMP4
      INTEGER            I, INFO, IY, J, JX, KX, KY, LENX, LENY, M4, N4
C     .. External Subroutines ..
      EXTERNAL           F06AAZ
C     .. Intrinsic Functions ..
      INTRINSIC          MAX
C     ..
C     .. Executable Statements ..
C
C     Test the input parameters.
C
      INFO = 0
      IF     ( .NOT.(TRANS.EQ.'N' .OR. TRANS.EQ.'n').AND.
     $         .NOT.(TRANS.EQ.'T' .OR. TRANS.EQ.'t').AND.
     $         .NOT.(TRANS.EQ.'C' .OR. TRANS.EQ.'c')      )THEN
         INFO = 1
      ELSE IF( M.LT.0 )THEN
         INFO = 2
      ELSE IF( N.LT.0 )THEN
         INFO = 3
      ELSE IF( LDA.LT.MAX( 1, M ) )THEN
         INFO = 6
      ELSE IF( INCX.EQ.0 )THEN
         INFO = 8
      ELSE IF( INCY.EQ.0 )THEN
         INFO = 11
      END IF
      IF( INFO.NE.0 )THEN
         CALL F06AAZ( 'F06PAF/DGEMV ', INFO )
         RETURN
      END IF
C
C     Quick return if possible.
C
      IF( ( M.EQ.0 ).OR.( N.EQ.0 ).OR.
     $    ( ( ALPHA.EQ.ZERO ).AND.( BETA.EQ.ONE ) ) )
     $   RETURN
C
C     Set  LENX  and  LENY, the lengths of the vectors x and y, and set
C     up the start points in  X  and  Y.
C
      IF( (TRANS.EQ.'N' .OR. TRANS.EQ.'n') )THEN
         LENX = N
         LENY = M
      ELSE
         LENX = M
         LENY = N
      END IF
      IF( INCX.GT.0 )THEN
         KX = 1
      ELSE
         KX = 1 - ( LENX - 1 )*INCX
      END IF
      IF( INCY.GT.0 )THEN
         KY = 1
      ELSE
         KY = 1 - ( LENY - 1 )*INCY
      END IF
C
C     Start the operations. In this version the inner loops are all
C     equivalent to AXPY operations.
C
C     First form  y := beta*y.
C
      IF( BETA.NE.ONE )THEN
         IF( INCY.EQ.1 )THEN
            IF( BETA.EQ.ZERO )THEN
               DO 10, I = 1, LENY
                  Y( I ) = ZERO
   10          CONTINUE
            ELSE
               DO 20, I = 1, LENY
                  Y( I ) = BETA*Y( I )
   20          CONTINUE
            END IF
         ELSE
            IY = KY
            IF( BETA.EQ.ZERO )THEN
               DO 30, I = 1, LENY
                  Y( IY ) = ZERO
                  IY      = IY   + INCY
   30          CONTINUE
            ELSE
               DO 40, I = 1, LENY
                  Y( IY ) = BETA*Y( IY )
                  IY      = IY           + INCY
   40          CONTINUE
            END IF
         END IF
      END IF
      IF( ALPHA.EQ.ZERO )
     $   RETURN
      JX = KX
      IF( (TRANS.EQ.'N' .OR. TRANS.EQ.'n') )THEN
C
C        Form  y := alpha*A*x + y.
C
         IF( INCY.EQ.1 )THEN
C**** U n r o l l   t o   d e p t h   4 ********************************
            N4 = 4*( N/4 )
            DO 60, J = 1, N4, 4
               TEMP1 = ALPHA*X( JX )
               TEMP2 = ALPHA*X( JX + INCX )
               TEMP3 = ALPHA*X( JX + 2*INCX )
               TEMP4 = ALPHA*X( JX + 3*INCX )
               IF( TEMP1.NE.ZERO.OR.TEMP2.NE.ZERO.OR.TEMP3.NE.ZERO.OR.
     $             TEMP4.NE.ZERO )THEN
                  DO 50, I = 1, M
                     Y( I ) = ( ( ( ( Y( I ) + TEMP1*A( I, J ) )
     $                        + TEMP2*A( I, J + 1 ) )
     $                        + TEMP3*A( I, J + 2 ) )
     $                        + TEMP4*A( I, J + 3 ) )
   50             CONTINUE
               END IF
               JX = JX + 4*INCX
   60       CONTINUE
C**** Clean-up loop ****************************************************
            DO 80, J = N4 + 1, N, 1
               TEMP = ALPHA*X( JX )
               IF( TEMP.NE.ZERO )THEN
                  DO 70, I = 1, M
                     Y( I ) = Y( I ) + TEMP*A( I, J )
   70             CONTINUE
               END IF
               JX = JX + INCX
   80       CONTINUE
         ELSE
C**** U n r o l l   t o   d e p t h   4 ********************************
            N4 = 4*( N/4 )
            DO 100, J = 1, N4, 4
               TEMP1 = ALPHA*X( JX )
               TEMP2 = ALPHA*X( JX + INCX )
               TEMP3 = ALPHA*X( JX + 2*INCX )
               TEMP4 = ALPHA*X( JX + 3*INCX )
               IF( TEMP1.NE.ZERO.OR.TEMP2.NE.ZERO.OR.TEMP3.NE.ZERO.OR.
     $             TEMP4.NE.ZERO )THEN
                  IY = KY
                  DO 90, I = 1, M
                     Y( IY ) = ( ( ( ( Y( IY ) + TEMP1*A( I, J ) )
     $                         + TEMP2*A( I, J + 1 ) )
     $                         + TEMP3*A( I, J + 2 ) )
     $                         + TEMP4*A( I, J + 3 ) )
                     IY = IY + INCY
   90             CONTINUE
               END IF
               JX = JX + 4*INCX
  100       CONTINUE
C**** Clean-up loop ****************************************************
            DO 120, J = N4 + 1, N, 1
               IF( X( JX ).NE.ZERO )THEN
                  TEMP = ALPHA*X( JX )
                  IY = KY
                  DO 110, I = 1, M
                     Y( IY ) = Y( IY ) + TEMP*A( I, J )
                     IY = IY + INCY
  110             CONTINUE
               END IF
               JX = JX + INCX
  120       CONTINUE
         END IF
      ELSE
C
C        Form  y := alpha*A'*x + y.
C
         IF( INCY.EQ.1 )THEN
C**** U n r o l l   t o   d e p t h   4 ********************************
            M4 = 4*( M/4 )
            DO 140, J = 1, M4, 4
               TEMP1 = ALPHA*X( JX )
               TEMP2 = ALPHA*X( JX + INCX )
               TEMP3 = ALPHA*X( JX + 2*INCX )
               TEMP4 = ALPHA*X( JX + 3*INCX )
               IF( TEMP1.NE.ZERO.OR.TEMP2.NE.ZERO.OR.TEMP3.NE.ZERO.OR.
     $             TEMP4.NE.ZERO )THEN
                  DO 130, I = 1, N
                     Y( I ) = ( ( ( ( Y( I ) + TEMP1*A( J, I ) )
     $                        + TEMP2*A( J + 1, I ) )
     $                        + TEMP3*A( J + 2, I ) )
     $                        + TEMP4*A( J + 3, I ) )
  130             CONTINUE
               END IF
               JX = JX + 4*INCX
  140       CONTINUE
C**** Clean-up loop ****************************************************
            DO 160, J = M4 + 1, M, 1
               TEMP = ALPHA*X( JX )
               IF( TEMP.NE.ZERO )THEN
                  DO 150, I = 1, N
                     Y( I ) = Y( I ) + TEMP*A( J, I )
  150             CONTINUE
               END IF
               JX = JX + INCX
  160       CONTINUE
         ELSE
C**** U n r o l l   t o   d e p t h   4 ********************************
            M4 = 4*( M/4 )
            DO 180, J = 1, M4, 4
               TEMP1 = ALPHA*X( JX )
               TEMP2 = ALPHA*X( JX + INCX )
               TEMP3 = ALPHA*X( JX + 2*INCX )
               TEMP4 = ALPHA*X( JX + 3*INCX )
               IF( TEMP1.NE.ZERO.OR.TEMP2.NE.ZERO.OR.TEMP3.NE.ZERO.OR.
     $             TEMP4.NE.ZERO )THEN
                  IY = KY
                  DO 170, I = 1, N
                     Y( IY ) = ( ( ( ( Y( IY ) + TEMP1*A( J, I ) )
     $                         + TEMP2*A( J + 1, I ) )
     $                         + TEMP3*A( J + 2, I ) )
     $                         + TEMP4*A( J + 3, I ) )
                     IY = IY + INCY
  170             CONTINUE
               END IF
               JX = JX + 4*INCX
  180       CONTINUE
C**** Clean-up loop ****************************************************
            DO 200, J = M4 + 1, M, 1
               IF( X( JX ).NE.ZERO )THEN
                  TEMP = ALPHA*X( JX )
                  IY = KY
                  DO 190, I = 1, N
                     Y( IY ) = Y( IY ) + TEMP*A( J, I )
                     IY = IY + INCY
  190             CONTINUE
               END IF
               JX = JX + INCX
  200       CONTINUE
         END IF
      END IF
C
      RETURN
C
C     End of F06PAF (DGEMV ).
C
      END
      SUBROUTINE F06PCF( UPLO, N, ALPHA, A, LDA, X, INCX,
     $                   BETA, Y, INCY )
C     MARK 12 RELEASE. NAG COPYRIGHT 1986.
C     DOT VERSION FOR VECTOR MACHINES
C     .. Entry Points ..
      ENTRY      DSYMV ( UPLO, N, ALPHA, A, LDA, X, INCX,
     $                   BETA, Y, INCY )
C     .. Scalar Arguments ..
      DOUBLE PRECISION   ALPHA, BETA
      INTEGER            INCX, INCY, LDA, N
      CHARACTER*1        UPLO
C     .. Array Arguments ..
      DOUBLE PRECISION   A( LDA, * ), X( * ), Y( * )
C     ..
C
C  Purpose
C  =======
C
C  DSYMV  performs the matrix-vector  operation
C
C     y := alpha*A*x + beta*y,
C
C  where alpha and beta are scalars, x and y are n element vectors and
C  A is an n by n symmetric matrix.
C
C  Parameters
C  ==========
C
C  UPLO   - CHARACTER*1.
C           On entry, UPLO specifies whether the upper or lower
C           triangular part of the array A is to be referenced as
C           follows:
C
C              UPLO = 'U' or 'u'   Only the upper triangular part of A
C                                  is to be referenced.
C
C              UPLO = 'L' or 'l'   Only the lower triangular part of A
C                                  is to be referenced.
C
C           Unchanged on exit.
C
C  N      - INTEGER.
C           On entry, N specifies the order of the matrix A.
C           N must be at least zero.
C           Unchanged on exit.
C
C  ALPHA  - DOUBLE PRECISION.
C           On entry, ALPHA specifies the scalar alpha.
C           Unchanged on exit.
C
C  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
C           Before entry with  UPLO = 'U' or 'u', the leading n by n
C           upper triangular part of the array A must contain the upper
C           triangular part of the symmetric matrix and the strictly
C           lower triangular part of A is not referenced.
C           Before entry with UPLO = 'L' or 'l', the leading n by n
C           lower triangular part of the array A must contain the lower
C           triangular part of the symmetric matrix and the strictly
C           upper triangular part of A is not referenced.
C           Unchanged on exit.
C
C  LDA    - INTEGER.
C           On entry, LDA specifies the first dimension of A as declared
C           in the calling (sub) program. LDA must be at least
C           max( 1, n ).
C           Unchanged on exit.
C
C  X      - DOUBLE PRECISION array of dimension at least
C           ( 1 + ( n - 1 )*abs( INCX ) ).
C           Before entry, the incremented array X must contain the n
C           element vector x.
C           Unchanged on exit.
C
C  INCX   - INTEGER.
C           On entry, INCX specifies the increment for the elements of
C           X. INCX must not be zero.
C           Unchanged on exit.
C
C  BETA   - DOUBLE PRECISION.
C           On entry, BETA specifies the scalar beta. When BETA is
C           supplied as zero then Y need not be set on input.
C           Unchanged on exit.
C
C  Y      - DOUBLE PRECISION array of dimension at least
C           ( 1 + ( n - 1 )*abs( INCY ) ).
C           Before entry, the incremented array Y must contain the n
C           element vector y. On exit, Y is overwritten by the updated
C           vector y.
C
C  INCY   - INTEGER.
C           On entry, INCY specifies the increment for the elements of
C           Y. INCY must not be zero.
C           Unchanged on exit.
C
C
C  Level 2 Blas routine.
C
C  -- Written on 22-October-1986.
C     Jack Dongarra, Argonne National Lab.
C     Jeremy Du Croz, Nag Central Office.
C     Sven Hammarling, Nag Central Office.
C     Richard Hanson, Sandia National Labs.
C
C
C     .. Parameters ..
      DOUBLE PRECISION   ONE         , ZERO
      PARAMETER        ( ONE = 1.0D+0, ZERO = 0.0D+0 )
C     .. Local Scalars ..
      DOUBLE PRECISION   TEMP
      INTEGER            I, INFO, IX, IY, J, JX, JY, KX, KY
C     .. External Subroutines ..
      EXTERNAL           F06AAZ
C     .. Intrinsic Functions ..
      INTRINSIC          MAX
C     ..
C     .. Executable Statements ..
C
C     Test the input parameters.
C
      INFO = 0
      IF     ( .NOT.(UPLO.EQ.'U' .OR. UPLO.EQ.'u').AND.
     $         .NOT.(UPLO.EQ.'L' .OR. UPLO.EQ.'l')      )THEN
         INFO = 1
      ELSE IF( N.LT.0 )THEN
         INFO = 2
      ELSE IF( LDA.LT.MAX( 1, N ) )THEN
         INFO = 5
      ELSE IF( INCX.EQ.0 )THEN
         INFO = 7
      ELSE IF( INCY.EQ.0 )THEN
         INFO = 10
      END IF
      IF( INFO.NE.0 )THEN
         CALL F06AAZ( 'F06PCF/DSYMV ', INFO )
         RETURN
      END IF
C
C     Quick return if possible.
C
      IF( ( N.EQ.0 ).OR.( ( ALPHA.EQ.ZERO ).AND.( BETA.EQ.ONE ) ) )
     $   RETURN
C
C     Set up the start points in  X  and  Y.
C
      IF( INCX.GT.0 )THEN
         KX = 1
      ELSE
         KX = 1 - ( N - 1 )*INCX
      END IF
      IF( INCY.GT.0 )THEN
         KY = 1
      ELSE
         KY = 1 - ( N - 1 )*INCY
      END IF
C
C     Start the operations. In this version the inner loops are all
C     equivalent to dot-product operations.
C
C     First form  y := beta*y.
C
      IF( BETA.NE.ONE )THEN
         IF( INCY.EQ.1 )THEN
            IF( BETA.EQ.ZERO )THEN
               DO 10, I = 1, N
                  Y( I ) = ZERO
   10          CONTINUE
            ELSE
               DO 20, I = 1, N
                  Y( I ) = BETA*Y( I )
   20          CONTINUE
            END IF
         ELSE
            IY = KY
            IF( BETA.EQ.ZERO )THEN
               DO 30, I = 1, N
                  Y( IY ) = ZERO
                  IY      = IY   + INCY
   30          CONTINUE
            ELSE
               DO 40, I = 1, N
                  Y( IY ) = BETA*Y( IY )
                  IY      = IY           + INCY
   40          CONTINUE
            END IF
         END IF
      END IF
      IF( ALPHA.EQ.ZERO )
     $   RETURN
      JY = KY
      IF( (UPLO.EQ.'U' .OR. UPLO.EQ.'u') )THEN
C
C        Form  y  when A is stored in upper triangle.
C
         IF( INCX.EQ.1 )THEN
            DO 60, J = 1, N
               TEMP = ZERO
               DO 50, I = 1, J
                  TEMP = TEMP + A( I, J )*X( I )
   50          CONTINUE
               DO 55, I = J + 1, N
                  TEMP = TEMP + A( J, I )*X( I )
   55          CONTINUE
               Y(JY) = Y( JY ) + ALPHA*TEMP
               JY    = JY      + INCY
   60       CONTINUE
         ELSE
            JX = KX
            DO 80, J = 1, N
               TEMP = ZERO
               IX   = KX
               DO 70, I = 1, J
                  TEMP = TEMP + A( I, J )*X( IX )
                  IX   = IX   + INCX
   70          CONTINUE
               IX = JX + INCX
               DO 75, I = J + 1, N
                  TEMP = TEMP + A( J, I )*X( IX )
                  IX   = IX   + INCX
   75          CONTINUE
               Y( JY ) = Y( JY ) + ALPHA*TEMP
               JY      = JY      + INCY
               JX      = JX      + INCX
   80       CONTINUE
         END IF
      ELSE
C
C        Form  y  when A is stored in lower triangle.
C
         IF( INCX.EQ.1 )THEN
            DO 100, J = 1, N
               TEMP = ZERO
               DO 85, I = 1, J - 1
                  TEMP = TEMP + A( J, I )*X( I )
   85          CONTINUE
               DO 90, I = J, N
                  TEMP = TEMP + A( I, J )*X( I )
   90          CONTINUE
               Y( JY ) = Y( JY ) + ALPHA*TEMP
               JY      = JY      + INCY
  100       CONTINUE
         ELSE
            JX = KX
            DO 120, J = 1, N
               TEMP = ZERO
               IX   = KX
               DO 105, I = 1, J - 1
                  TEMP = TEMP + A( J, I )*X( IX )
                  IX   = IX   + INCX
  105          CONTINUE
               IX = JX
               DO 110, I = J, N
                  TEMP = TEMP + A( I, J )*X( IX )
                  IX   = IX   + INCX
  110          CONTINUE
               Y( JY ) = Y( JY ) + ALPHA*TEMP
               JY      = JY      + INCY
               JX      = JX      + INCX
  120       CONTINUE
         END IF
      END IF
C
      RETURN
C
C     End of F06PCF (DSYMV ).
C
      END
      SUBROUTINE F06PJF( UPLO, TRANS, DIAG, N, A, LDA, X, INCX )
C     MARK 12 RELEASE. NAG COPYRIGHT 1986.
C     AXP4 VERSION FOR VECTOR MACHINES
C     .. Entry Points ..
      ENTRY      DTRSV ( UPLO, TRANS, DIAG, N, A, LDA, X, INCX )
C     .. Scalar Arguments ..
      INTEGER            INCX, LDA, N
      CHARACTER*1        DIAG, TRANS, UPLO
C     .. Array Arguments ..
      DOUBLE PRECISION   A( LDA, * ), X( * )
C     ..
C
C  Purpose
C  =======
C
C  DTRSV  solves one of the systems of equations
C
C     A*x = b,   or   A'*x = b,
C
C  where b and x are n element vectors and A is an n by n unit, or
C  non-unit, upper or lower triangular matrix.
C
C  No test for singularity or near-singularity is included in this
C  routine. Such tests must be performed before calling this routine.
C
C  Parameters
C  ==========
C
C  UPLO   - CHARACTER*1.
C           On entry, UPLO specifies whether the matrix is an upper or
C           lower triangular matrix as follows:
C
C              UPLO = 'U' or 'u'   A is an upper triangular matrix.
C
C              UPLO = 'L' or 'l'   A is a lower triangular matrix.
C
C           Unchanged on exit.
C
C  TRANS  - CHARACTER*1.
C           On entry, TRANS specifies the equations to be solved as
C           follows:
C
C              TRANS = 'N' or 'n'   A*x = b.
C
C              TRANS = 'T' or 't'   A'*x = b.
C
C              TRANS = 'C' or 'c'   A'*x = b.
C
C           Unchanged on exit.
C
C  DIAG   - CHARACTER*1.
C           On entry, DIAG specifies whether or not A is unit
C           triangular as follows:
C
C              DIAG = 'U' or 'u'   A is assumed to be unit triangular.
C
C              DIAG = 'N' or 'n'   A is not assumed to be unit
C                                  triangular.
C
C           Unchanged on exit.
C
C  N      - INTEGER.
C           On entry, N specifies the order of the matrix A.
C           N must be at least zero.
C           Unchanged on exit.
C
C  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
C           Before entry with  UPLO = 'U' or 'u', the leading n by n
C           upper triangular part of the array A must contain the upper
C           triangular matrix and the strictly lower triangular part of
C           A is not referenced.
C           Before entry with UPLO = 'L' or 'l', the leading n by n
C           lower triangular part of the array A must contain the lower
C           triangular matrix and the strictly upper triangular part of
C           A is not referenced.
C           Note that when  DIAG = 'U' or 'u', the diagonal elements of
C           A are not referenced either, but are assumed to be unity.
C           Unchanged on exit.
C
C  LDA    - INTEGER.
C           On entry, LDA specifies the first dimension of A as declared
C           in the calling (sub) program. LDA must be at least
C           max( 1, n ).
C           Unchanged on exit.
C
C  X      - DOUBLE PRECISION array of dimension at least
C           ( 1 + ( n - 1 )*abs( INCX ) ).
C           Before entry, the incremented array X must contain the n
C           element right-hand side vector b. On exit, X is overwritten
C           with the solution vector x.
C
C  INCX   - INTEGER.
C           On entry, INCX specifies the increment for the elements of
C           X. INCX must not be zero.
C           Unchanged on exit.
C
C
C  Level 2 Blas routine.
C
C  -- Written on 22-October-1986.
C     Jack Dongarra, Argonne National Lab.
C     Jeremy Du Croz, Nag Central Office.
C     Sven Hammarling, Nag Central Office.
C     Richard Hanson, Sandia National Labs.
C  -- DO-loops unrolled on 20-November-1986.
C     Peter Mayes, Nag Central Office.
C
C
C     .. Parameters ..
      DOUBLE PRECISION   ZERO
      PARAMETER        ( ZERO = 0.0D+0 )
C     .. Local Scalars ..
      DOUBLE PRECISION   TEMP1, TEMP2, TEMP3, TEMP4
      INTEGER            I, INFO, IX, J, JX, KX, N4
      LOGICAL            NOUNIT
C     .. External Subroutines ..
      EXTERNAL           F06AAZ
C     .. Intrinsic Functions ..
      INTRINSIC          MAX, MOD
C     ..
C     .. Executable Statements ..
C
C     Test the input parameters.
C
      INFO = 0
      IF     ( .NOT.(UPLO .EQ.'U' .OR. UPLO .EQ.'u').AND.
     $         .NOT.(UPLO .EQ.'L' .OR. UPLO .EQ.'l')      )THEN
         INFO = 1
      ELSE IF( .NOT.(TRANS.EQ.'N' .OR. TRANS.EQ.'n').AND.
     $         .NOT.(TRANS.EQ.'T' .OR. TRANS.EQ.'t').AND.
     $         .NOT.(TRANS.EQ.'C' .OR. TRANS.EQ.'c')      )THEN
         INFO = 2
      ELSE IF( .NOT.(DIAG .EQ.'U' .OR. DIAG .EQ.'u').AND.
     $         .NOT.(DIAG .EQ.'N' .OR. DIAG .EQ.'n')      )THEN
         INFO = 3
      ELSE IF( N.LT.0 )THEN
         INFO = 4
      ELSE IF( LDA.LT.MAX( 1, N ) )THEN
         INFO = 6
      ELSE IF( INCX.EQ.0 )THEN
         INFO = 8
      END IF
      IF( INFO.NE.0 )THEN
         CALL F06AAZ( 'F06PJF/DTRSV ', INFO )
         RETURN
      END IF
C
C     Quick return if possible.
C
      IF( N.EQ.0 )
     $   RETURN
C
      NOUNIT = (DIAG.EQ.'N' .OR. DIAG.EQ.'n')
C
C     Set up the start point in X if the increment is not unity. This
C     will be  ( N - 1 )*INCX  too small for descending loops.
C
      IF( INCX.LE.0 )THEN
         KX = 1 - ( N - 1 )*INCX
      ELSE IF( INCX.NE.1 )THEN
         KX = 1
      END IF
C
C     Start the operations. In this version the inner loops are all
C     equivalent to AXPY operations.
C
      IF( (TRANS.EQ.'N' .OR. TRANS.EQ.'n') )THEN
C
C        Form  x := inv( A )*x.
C
         IF( (UPLO.EQ.'U' .OR. UPLO.EQ.'u') )THEN
            IF( INCX.EQ.1 )THEN
C**** U n r o l l   t o   D e p t h  4 ********** L O O P  A ***********
               N4 = MOD( N, 4 ) + 1
               DO 20, J = N, N4, -4
C**** 4 by 4 triangle at bottom of block *******************************
                  IF( NOUNIT )
     $               X( J ) = X( J )/A( J, J )
                  X( J - 1 ) = X( J - 1 ) - X( J )*A( J - 1, J )
                  IF( NOUNIT )
     $               X( J - 1 ) = X( J - 1 )/A( J - 1, J - 1 )
                  X( J - 2 ) = X( J - 2 ) - X( J )*A( J - 2, J ) -
     $                         X( J - 1 )*A( J - 2, J - 1 )
                  IF( NOUNIT )
     $               X( J - 2 ) = X( J - 2 )/A( J - 2, J - 2 )
                  X( J - 3 ) = X( J - 3 ) - X( J )*A( J - 3, J ) -
     $                         X( J - 1 )*A( J - 3, J - 1 ) - X( J - 2 )
     $                         *A( J - 3, J - 2 )
                  IF( NOUNIT )
     $               X( J - 3 ) = X( J - 3 )/A( J - 3, J - 3 )
C**** Unrolled main loop ***********************************************
                  TEMP1 = X( J )
                  TEMP2 = X( J - 1 )
                  TEMP3 = X( J - 2 )
                  TEMP4 = X( J - 3 )
                  IF( TEMP1.NE.ZERO.OR.TEMP2.NE.ZERO.OR.TEMP3.NE.
     $                ZERO.OR.TEMP4.NE.ZERO )THEN
                     DO 10, I = J - 4, 1, -1
                        X( I ) = ( ( ( ( X( I ) - TEMP1*A( I, J ) )
     $                           - TEMP2*A( I, J - 1 ) )
     $                           - TEMP3*A( I, J - 2 ) )
     $                           - TEMP4*A( I, J - 3 ) )
   10                CONTINUE
                  END IF
   20          CONTINUE
C**** Left-overs on top left corner ************************************
               IF( N4.GE.2 )THEN
                  IF( NOUNIT )
     $               X( N4 - 1 ) = X( N4 - 1 )/A( N4 - 1, N4 - 1 )
               END IF
               IF( N4.GE.3 )THEN
                  X( N4 - 2 ) = X( N4 - 2 ) - X( N4 - 1 )
     $                          *A( N4 - 2, N4 - 1 )
                  IF( NOUNIT )
     $               X( N4 - 2 ) = X( N4 - 2 )/A( N4 - 2, N4 - 2 )
               END IF
               IF( N4.GE.4 )THEN
                  X( N4 - 3 ) = X( N4 - 3 ) - X( N4 - 1 )
     $                          *A( N4 - 3, N4 - 1 ) - X( N4 - 2 )
     $                          *A( N4 - 3, N4 - 2 )
                  IF( NOUNIT )
     $               X( N4 - 3 ) = X( N4 - 3 )/A( N4 - 3, N4 - 3 )
               END IF
            ELSE
               JX = KX + ( N - 1 )*INCX
C**** U n r o l l   t o   D e p t h  4 ********** L O O P  B ***********
               N4 = MOD( N, 4 ) + 1
               DO 40, J = N, N4, -4
C**** 4 by 4 triangle at bottom of block *******************************
                  IF( NOUNIT )
     $               X( JX ) = X( JX )/A( J, J )
                  X( JX - INCX ) = X( JX - INCX ) - X( JX )
     $                             *A( J - 1, J )
                  IF( NOUNIT )
     $               X( JX - INCX ) = X( JX - INCX )/A( J - 1, J - 1 )
                  X( JX - 2*INCX ) = X( JX - 2*INCX ) - X( JX )
     $                               *A( J - 2, J ) - X( JX - INCX )
     $                               *A( J - 2, J - 1 )
                  IF( NOUNIT )
     $               X( JX - 2*INCX ) = X( JX - 2*INCX )
     $                                  /A( J - 2, J - 2 )
                  X( JX - 3*INCX ) = X( JX - 3*INCX ) - X( JX )
     $                               *A( J - 3, J ) - X( JX - INCX )
     $                               *A( J - 3, J - 1 ) -
     $                               X( JX - 2*INCX )*A( J - 3, J - 2 )
                  IF( NOUNIT )
     $               X( JX - 3*INCX ) = X( JX - 3*INCX )
     $                                  /A( J - 3, J - 3 )
C**** Unrolled main loop ***********************************************
                  TEMP1 = X( JX )
                  TEMP2 = X( JX - INCX )
                  TEMP3 = X( JX - 2*INCX )
                  TEMP4 = X( JX - 3*INCX )
                  IF( TEMP1.NE.ZERO.OR.TEMP2.NE.ZERO.OR.TEMP3.NE.
     $                ZERO.OR.TEMP4.NE.ZERO )THEN
                     IX = JX - 3*INCX
                     DO 30, I = J - 4, 1, -1
                        IX = IX - INCX
                        X( IX ) = ( ( ( ( X( IX ) - TEMP1*A( I, J ) )
     $                            - TEMP2*A( I, J - 1 ) )
     $                            - TEMP3*A( I, J - 2 ) )
     $                            - TEMP4*A( I, J - 3 ) )
   30                CONTINUE
                  END IF
                  JX = JX - 4*INCX
   40          CONTINUE
C**** Left-overs on top left corner ************************************
               IF( N4.GE.2 )THEN
                  IF( NOUNIT )
     $               X( JX ) = X( JX )/A( N4 - 1, N4 - 1 )
               END IF
               IF( N4.GE.3 )THEN
                  X( JX - INCX ) = X( JX - INCX ) - X( JX )
     $                             *A( N4 - 2, N4 - 1 )
                  IF( NOUNIT )
     $               X( JX - INCX ) = X( JX - INCX )/A( N4 - 2, N4 - 2 )
               END IF
               IF( N4.GE.4 )THEN
                  X( JX - 2*INCX ) = X( JX - 2*INCX ) - X( JX )
     $                               *A( N4 - 3, N4 - 1 ) -
     $                               X( JX - INCX )*A( N4 - 3, N4 - 2 )
                  IF( NOUNIT )
     $               X( JX - 2*INCX ) = X( JX - 2*INCX )
     $                                  /A( N4 - 3, N4 - 3 )
               END IF
            END IF
         ELSE
            IF( INCX.EQ.1 )THEN
C**** U n r o l l   t o   D e p t h  4 ********** L O O P  C ***********
               N4 = 4*( N/4 )
               DO 60, J = 1, N4, 4
C**** 4 by 4 triangle at bottom of block *******************************
                  IF( NOUNIT )
     $               X( J ) = X( J )/A( J, J )
                  X( J + 1 ) = X( J + 1 ) - X( J )*A( J + 1, J )
                  IF( NOUNIT )
     $               X( J + 1 ) = X( J + 1 )/A( J + 1, J + 1 )
                  X( J + 2 ) = X( J + 2 ) - X( J )*A( J + 2, J ) -
     $                         X( J + 1 )*A( J + 2, J + 1 )
                  IF( NOUNIT )
     $               X( J + 2 ) = X( J + 2 )/A( J + 2, J + 2 )
                  X( J + 3 ) = X( J + 3 ) - X( J )*A( J + 3, J ) -
     $                         X( J + 1 )*A( J + 3, J + 1 ) - X( J + 2 )
     $                         *A( J + 3, J + 2 )
                  IF( NOUNIT )
     $               X( J + 3 ) = X( J + 3 )/A( J + 3, J + 3 )
C**** Unrolled main loop ***********************************************
                  TEMP1 = X( J )
                  TEMP2 = X( J + 1 )
                  TEMP3 = X( J + 2 )
                  TEMP4 = X( J + 3 )
                  IF( TEMP1.NE.ZERO.OR.TEMP2.NE.ZERO.OR.TEMP3.NE.
     $                ZERO.OR.TEMP4.NE.ZERO )THEN
                     DO 50, I = J + 4, N
                        X( I ) = ( ( ( ( X( I ) - TEMP1*A( I, J ) )
     $                           - TEMP2*A( I, J + 1 ) )
     $                           - TEMP3*A( I, J + 2 ) )
     $                           - TEMP4*A( I, J + 3 ) )
   50                CONTINUE
                  END IF
   60          CONTINUE
C**** Left-overs on top left corner ************************************
               IF( N4 + 1.LE.N )THEN
                  IF( NOUNIT )
     $               X( N4 + 1 ) = X( N4 + 1 )/A( N4 + 1, N4 + 1 )
               END IF
               IF( N4 + 2.LE.N )THEN
                  X( N4 + 2 ) = X( N4 + 2 ) - X( N4 + 1 )
     $                          *A( N4 + 2, N4 + 1 )
                  IF( NOUNIT )
     $               X( N4 + 2 ) = X( N4 + 2 )/A( N4 + 2, N4 + 2 )
               END IF
               IF( N4 + 3.LE.N )THEN
                  X( N4 + 3 ) = X( N4 + 3 ) - X( N4 + 1 )
     $                          *A( N4 + 3, N4 + 1 ) - X( N4 + 2 )
     $                          *A( N4 + 3, N4 + 2 )
                  IF( NOUNIT )
     $               X( N4 + 3 ) = X( N4 + 3 )/A( N4 + 3, N4 + 3 )
               END IF
            ELSE
               JX = KX
C**** U n r o l l   t o   D e p t h  4 ********** L O O P  D ***********
               N4 = 4*( N/4 )
               DO 80, J = 1, N4, 4
C**** 4 by 4 triangle at bottom of block *******************************
                  IF( NOUNIT )
     $               X( JX ) = X( JX )/A( J, J )
                  X( JX + INCX ) = X( JX + INCX ) - X( JX )
     $                             *A( J + 1, J )
                  IF( NOUNIT )
     $               X( JX + INCX ) = X( JX + INCX )/A( J + 1, J + 1 )
                  X( JX + 2*INCX ) = X( JX + 2*INCX ) - X( JX )
     $                               *A( J + 2, J ) - X( JX + INCX )
     $                               *A( J + 2, J + 1 )
                  IF( NOUNIT )
     $               X( JX + 2*INCX ) = X( JX + 2*INCX )
     $                                  /A( J + 2, J + 2 )
                  X( JX + 3*INCX ) = X( JX + 3*INCX ) - X( JX )
     $                               *A( J + 3, J ) - X( JX + INCX )
     $                               *A( J + 3, J + 1 ) -
     $                               X( JX + 2*INCX )*A( J + 3, J + 2 )
                  IF( NOUNIT )
     $               X( JX + 3*INCX ) = X( JX + 3*INCX )
     $                                  /A( J + 3, J + 3 )
C**** Unrolled main loop ***********************************************
                  TEMP1 = X( JX )
                  TEMP2 = X( JX + INCX )
                  TEMP3 = X( JX + 2*INCX )
                  TEMP4 = X( JX + 3*INCX )
                  IF( TEMP1.NE.ZERO.OR.TEMP2.NE.ZERO.OR.TEMP3.NE.
     $                ZERO.OR.TEMP4.NE.ZERO )THEN
                     IX = JX + 3*INCX
                     DO 70, I = J + 4, N
                        IX = IX + INCX
                        X( IX ) = ( ( ( ( X( IX ) - TEMP1*A( I, J ) )
     $                            - TEMP2*A( I, J + 1 ) )
     $                            - TEMP3*A( I, J + 2 ) )
     $                            - TEMP4*A( I, J + 3 ) )
   70                CONTINUE
                  END IF
                  JX = JX + 4*INCX
   80          CONTINUE
C**** Left-overs on top left corner ************************************
               IF( N4 + 1.LE.N )THEN
                  IF( NOUNIT )
     $               X( JX ) = X( JX )/A( N4 + 1, N4 + 1 )
               END IF
               IF( N4 + 2.LE.N )THEN
                  X( JX + INCX ) = X( JX + INCX ) - X( JX )
     $                             *A( N4 + 2, N4 + 1 )
                  IF( NOUNIT )
     $               X( JX + INCX ) = X( JX + INCX )/A( N4 + 2, N4 + 2 )
               END IF
               IF( N4 + 3.LE.N )THEN
                  X( JX + 2*INCX ) = X( JX + 2*INCX ) - X( JX )
     $                               *A( N4 + 3, N4 + 1 ) -
     $                               X( JX + INCX )*A( N4 + 3, N4 + 2 )
                  IF( NOUNIT )
     $               X( JX + 2*INCX ) = X( JX + 2*INCX )
     $                                  /A( N4 + 3, N4 + 3 )
               END IF
            END IF
         END IF
      ELSE
C
C        Form  x := inv( A' )*x.
C
         IF( (UPLO.EQ.'U' .OR. UPLO.EQ.'u') )THEN
            IF( INCX.EQ.1 )THEN
C**** U n r o l l   t o   D e p t h  4 ********** L O O P  E ***********
               N4 = 4*( N/4 )
               DO 100, J = 1, N4, 4
C**** 4 by 4 triangle at bottom of block *******************************
                  IF( NOUNIT )
     $               X( J ) = X( J )/A( J, J )
                  X( J + 1 ) = X( J + 1 ) - X( J )*A( J, J + 1 )
                  IF( NOUNIT )
     $               X( J + 1 ) = X( J + 1 )/A( J + 1, J + 1 )
                  X( J + 2 ) = X( J + 2 ) - X( J )*A( J, J + 2 ) -
     $                         X( J + 1 )*A( J + 1, J + 2 )
                  IF( NOUNIT )
     $               X( J + 2 ) = X( J + 2 )/A( J + 2, J + 2 )
                  X( J + 3 ) = X( J + 3 ) - X( J )*A( J, J + 3 ) -
     $                         X( J + 1 )*A( J + 1, J + 3 ) - X( J + 2 )
     $                         *A( J + 2, J + 3 )
                  IF( NOUNIT )
     $               X( J + 3 ) = X( J + 3 )/A( J + 3, J + 3 )
C**** Unrolled main loop ***********************************************
                  TEMP1 = X( J )
                  TEMP2 = X( J + 1 )
                  TEMP3 = X( J + 2 )
                  TEMP4 = X( J + 3 )
                  IF( TEMP1.NE.ZERO.OR.TEMP2.NE.ZERO.OR.TEMP3.NE.
     $                ZERO.OR.TEMP4.NE.ZERO )THEN
                     DO 90, I = J + 4, N
                        X( I ) = ( ( ( ( X( I ) - TEMP1*A( J, I ) )
     $                           - TEMP2*A( J + 1, I ) )
     $                           - TEMP3*A( J + 2, I ) )
     $                           - TEMP4*A( J + 3, I ) )
   90                CONTINUE
                  END IF
  100          CONTINUE
C**** Left-overs on top left corner ************************************
               IF( N4 + 1.LE.N )THEN
                  IF( NOUNIT )
     $               X( N4 + 1 ) = X( N4 + 1 )/A( N4 + 1, N4 + 1 )
               END IF
               IF( N4 + 2.LE.N )THEN
                  X( N4 + 2 ) = X( N4 + 2 ) - X( N4 + 1 )
     $                          *A( N4 + 1, N4 + 2 )
                  IF( NOUNIT )
     $               X( N4 + 2 ) = X( N4 + 2 )/A( N4 + 2, N4 + 2 )
               END IF
               IF( N4 + 3.LE.N )THEN
                  X( N4 + 3 ) = X( N4 + 3 ) - X( N4 + 1 )
     $                          *A( N4 + 1, N4 + 3 ) - X( N4 + 2 )
     $                          *A( N4 + 2, N4 + 3 )
                  IF( NOUNIT )
     $               X( N4 + 3 ) = X( N4 + 3 )/A( N4 + 3, N4 + 3 )
               END IF
            ELSE
               JX = KX
C**** U n r o l l   t o   D e p t h  4 ********** L O O P  D ***********
               N4 = 4*( N/4 )
               DO 120, J = 1, N4, 4
C**** 4 by 4 triangle at bottom of block *******************************
                  IF( NOUNIT )
     $               X( JX ) = X( JX )/A( J, J )
                  X( JX + INCX ) = X( JX + INCX ) - X( JX )
     $                             *A( J, J + 1 )
                  IF( NOUNIT )
     $               X( JX + INCX ) = X( JX + INCX )/A( J + 1, J + 1 )
                  X( JX + 2*INCX ) = X( JX + 2*INCX ) - X( JX )
     $                               *A( J, J + 2 ) - X( JX + INCX )
     $                               *A( J + 1, J + 2 )
                  IF( NOUNIT )
     $               X( JX + 2*INCX ) = X( JX + 2*INCX )
     $                                  /A( J + 2, J + 2 )
                  X( JX + 3*INCX ) = X( JX + 3*INCX ) - X( JX )
     $                               *A( J, J + 3 ) - X( JX + INCX )
     $                               *A( J + 1, J + 3 ) -
     $                               X( JX + 2*INCX )*A( J + 2, J + 3 )
                  IF( NOUNIT )
     $               X( JX + 3*INCX ) = X( JX + 3*INCX )
     $                                  /A( J + 3, J + 3 )
C**** Unrolled main loop ***********************************************
                  TEMP1 = X( JX )
                  TEMP2 = X( JX + INCX )
                  TEMP3 = X( JX + 2*INCX )
                  TEMP4 = X( JX + 3*INCX )
                  IF( TEMP1.NE.ZERO.OR.TEMP2.NE.ZERO.OR.TEMP3.NE.
     $                ZERO.OR.TEMP4.NE.ZERO )THEN
                     IX = JX + 3*INCX
                     DO 110, I = J + 4, N
                        IX = IX + INCX
                        X( IX ) = ( ( ( ( X( IX ) - TEMP1*A( J, I ) )
     $                            - TEMP2*A( J + 1, I ) )
     $                            - TEMP3*A( J + 2, I ) )
     $                            - TEMP4*A( J + 3, I ) )
  110                CONTINUE
                  END IF
                  JX = JX + 4*INCX
  120          CONTINUE
C**** Left-overs on top left corner ************************************
               IF( N4 + 1.LE.N )THEN
                  IF( NOUNIT )
     $               X( JX ) = X( JX )/A( N4 + 1, N4 + 1 )
               END IF
               IF( N4 + 2.LE.N )THEN
                  X( JX + INCX ) = X( JX + INCX ) - X( JX )
     $                             *A( N4 + 1, N4 + 2 )
                  IF( NOUNIT )
     $               X( JX + INCX ) = X( JX + INCX )/A( N4 + 2, N4 + 2 )
               END IF
               IF( N4 + 3.LE.N )THEN
                  X( JX + 2*INCX ) = X( JX + 2*INCX ) - X( JX )
     $                               *A( N4 + 1, N4 + 3 ) -
     $                               X( JX + INCX )*A( N4 + 2, N4 + 3 )
                  IF( NOUNIT )
     $               X( JX + 2*INCX ) = X( JX + 2*INCX )
     $                                  /A( N4 + 3, N4 + 3 )
               END IF
            END IF
         ELSE
            IF( INCX.EQ.1 )THEN
C**** U n r o l l   t o   D e p t h  4 ********** L O O P  G ***********
               N4 = MOD( N, 4 ) + 1
               DO 140, J = N, N4, -4
C**** 4 by 4 triangle at bottom of block *******************************
                  IF( NOUNIT )
     $               X( J ) = X( J )/A( J, J )
                  X( J - 1 ) = X( J - 1 ) - X( J )*A( J, J - 1 )
                  IF( NOUNIT )
     $               X( J - 1 ) = X( J - 1 )/A( J - 1, J - 1 )
                  X( J - 2 ) = X( J - 2 ) - X( J )*A( J, J - 2 ) -
     $                         X( J - 1 )*A( J - 1, J - 2 )
                  IF( NOUNIT )
     $               X( J - 2 ) = X( J - 2 )/A( J - 2, J - 2 )
                  X( J - 3 ) = X( J - 3 ) - X( J )*A( J, J - 3 ) -
     $                         X( J - 1 )*A( J - 1, J - 3 ) - X( J - 2 )
     $                         *A( J - 2, J - 3 )
                  IF( NOUNIT )
     $               X( J - 3 ) = X( J - 3 )/A( J - 3, J - 3 )
C**** Unrolled main loop ***********************************************
                  TEMP1 = X( J )
                  TEMP2 = X( J - 1 )
                  TEMP3 = X( J - 2 )
                  TEMP4 = X( J - 3 )
                  IF( TEMP1.NE.ZERO.OR.TEMP2.NE.ZERO.OR.TEMP3.NE.
     $                ZERO.OR.TEMP4.NE.ZERO )THEN
                     DO 130, I = J - 4, 1, -1
                        X( I ) = ( ( ( ( X( I ) - TEMP1*A( J, I ) )
     $                           - TEMP2*A( J - 1, I ) )
     $                           - TEMP3*A( J - 2, I ) )
     $                           - TEMP4*A( J - 3, I ) )
  130                CONTINUE
                  END IF
  140          CONTINUE
C**** Left-overs on top left corner ************************************
               IF( N4.GE.2 )THEN
                  IF( NOUNIT )
     $               X( N4 - 1 ) = X( N4 - 1 )/A( N4 - 1, N4 - 1 )
               END IF
               IF( N4.GE.3 )THEN
                  X( N4 - 2 ) = X( N4 - 2 ) - X( N4 - 1 )
     $                          *A( N4 - 1, N4 - 2 )
                  IF( NOUNIT )
     $               X( N4 - 2 ) = X( N4 - 2 )/A( N4 - 2, N4 - 2 )
               END IF
               IF( N4.GE.4 )THEN
                  X( N4 - 3 ) = X( N4 - 3 ) - X( N4 - 1 )
     $                          *A( N4 - 1, N4 - 3 ) - X( N4 - 2 )
     $                          *A( N4 - 2, N4 - 3 )
                  IF( NOUNIT )
     $               X( N4 - 3 ) = X( N4 - 3 )/A( N4 - 3, N4 - 3 )
               END IF
            ELSE
               JX = KX + ( N - 1 )*INCX
C**** U n r o l l   t o   D e p t h  4 ********** L O O P  B ***********
               N4 = MOD( N, 4 ) + 1
               DO 160, J = N, N4, -4
C**** 4 by 4 triangle at bottom of block *******************************
                  IF( NOUNIT )
     $               X( JX ) = X( JX )/A( J, J )
                  X( JX - INCX ) = X( JX - INCX ) - X( JX )
     $                             *A( J, J - 1 )
                  IF( NOUNIT )
     $               X( JX - INCX ) = X( JX - INCX )/A( J - 1, J - 1 )
                  X( JX - 2*INCX ) = X( JX - 2*INCX ) - X( JX )
     $                               *A( J, J - 2 ) - X( JX - INCX )
     $                               *A( J - 1, J - 2 )
                  IF( NOUNIT )
     $               X( JX - 2*INCX ) = X( JX - 2*INCX )
     $                                  /A( J - 2, J - 2 )
                  X( JX - 3*INCX ) = X( JX - 3*INCX ) - X( JX )
     $                               *A( J, J - 3 ) - X( JX - INCX )
     $                               *A( J - 1, J - 3 ) -
     $                               X( JX - 2*INCX )*A( J - 2, J - 3 )
                  IF( NOUNIT )
     $               X( JX - 3*INCX ) = X( JX - 3*INCX )
     $                                  /A( J - 3, J - 3 )
C**** Unrolled main loop ***********************************************
                  TEMP1 = X( JX )
                  TEMP2 = X( JX - INCX )
                  TEMP3 = X( JX - 2*INCX )
                  TEMP4 = X( JX - 3*INCX )
                  IF( TEMP1.NE.ZERO.OR.TEMP2.NE.ZERO.OR.TEMP3.NE.
     $                ZERO.OR.TEMP4.NE.ZERO )THEN
                     IX = JX - 3*INCX
                     DO 150, I = J - 4, 1, -1
                        IX = IX - INCX
                        X( IX ) = ( ( ( ( X( IX ) - TEMP1*A( J, I ) )
     $                            - TEMP2*A( J - 1, I ) )
     $                            - TEMP3*A( J - 2, I ) )
     $                            - TEMP4*A( J - 3, I ) )
  150                CONTINUE
                  END IF
                  JX = JX - 4*INCX
  160          CONTINUE
C**** Left-overs on top left corner ************************************
               IF( N4.GE.2 )THEN
                  IF( NOUNIT )
     $               X( JX ) = X( JX )/A( N4 - 1, N4 - 1 )
               END IF
               IF( N4.GE.3 )THEN
                  X( JX - INCX ) = X( JX - INCX ) - X( JX )
     $                             *A( N4 - 1, N4 - 2 )
                  IF( NOUNIT )
     $               X( JX - INCX ) = X( JX - INCX )/A( N4 - 2, N4 - 2 )
               END IF
               IF( N4.GE.4 )THEN
                  X( JX - 2*INCX ) = X( JX - 2*INCX ) - X( JX )
     $                               *A( N4 - 1, N4 - 3 ) -
     $                               X( JX - INCX )*A( N4 - 2, N4 - 3 )
                  IF( NOUNIT )
     $               X( JX - 2*INCX ) = X( JX - 2*INCX )
     $                                  /A( N4 - 3, N4 - 3 )
               END IF
            END IF
         END IF
      END IF
C
      RETURN
C
C     End of F06PJF (DTRSV ).
C
      END
      SUBROUTINE F06PMF( M, N, ALPHA, X, INCX, Y, INCY, A, LDA )
C     MARK 13 RE-ISSUE. NAG COPYRIGHT 1988.
C     .. Entry Points ..
      ENTRY      DGER  ( M, N, ALPHA, X, INCX, Y, INCY, A, LDA )
C     .. Scalar Arguments ..
      DOUBLE PRECISION   ALPHA
      INTEGER            INCX, INCY, LDA, M, N
C     .. Array Arguments ..
      DOUBLE PRECISION   A( LDA, * ), X( * ), Y( * )
C     ..
C
C  Purpose
C  =======
C
C  DGER   performs the rank 1 operation
C
C     A := alpha*x*y' + A,
C
C  where alpha is a scalar, x is an m element vector, y is an n element
C  vector and A is an m by n matrix.
C
C  Parameters
C  ==========
C
C  M      - INTEGER.
C           On entry, M specifies the number of rows of the matrix A.
C           M must be at least zero.
C           Unchanged on exit.
C
C  N      - INTEGER.
C           On entry, N specifies the number of columns of the matrix A.
C           N must be at least zero.
C           Unchanged on exit.
C
C  ALPHA  - DOUBLE PRECISION.
C           On entry, ALPHA specifies the scalar alpha.
C           Unchanged on exit.
C
C  X      - DOUBLE PRECISION array of dimension at least
C           ( 1 + ( m - 1 )*abs( INCX ) ).
C           Before entry, the incremented array X must contain the m
C           element vector x.
C           Unchanged on exit.
C
C  INCX   - INTEGER.
C           On entry, INCX specifies the increment for the elements of
C           X. INCX must not be zero.
C           Unchanged on exit.
C
C  Y      - DOUBLE PRECISION array of dimension at least
C           ( 1 + ( n - 1 )*abs( INCY ) ).
C           Before entry, the incremented array Y must contain the n
C           element vector y.
C           Unchanged on exit.
C
C  INCY   - INTEGER.
C           On entry, INCY specifies the increment for the elements of
C           Y. INCY must not be zero.
C           Unchanged on exit.
C
C  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
C           Before entry, the leading m by n part of the array A must
C           contain the matrix of coefficients. On exit, A is
C           overwritten by the updated matrix.
C
C  LDA    - INTEGER.
C           On entry, LDA specifies the first dimension of A as declared
C           in the calling (sub) program. LDA must be at least
C           max( 1, m ).
C           Unchanged on exit.
C
C
C  Level 2 Blas routine.
C
C  -- Written on 22-October-1986.
C     Jack Dongarra, Argonne National Lab.
C     Jeremy Du Croz, Nag Central Office.
C     Sven Hammarling, Nag Central Office.
C     Richard Hanson, Sandia National Labs.
C
C
C     .. Parameters ..
      DOUBLE PRECISION   ZERO
      PARAMETER        ( ZERO = 0.0D+0 )
C     .. Local Scalars ..
      DOUBLE PRECISION   TEMP
      INTEGER            I, INFO, IX, J, JY, KX
C     .. External Subroutines ..
      EXTERNAL           F06AAZ
C     .. Intrinsic Functions ..
      INTRINSIC          MAX
C     ..
C     .. Executable Statements ..
C
C     Test the input parameters.
C
      INFO = 0
      IF     ( M.LT.0 )THEN
         INFO = 1
      ELSE IF( N.LT.0 )THEN
         INFO = 2
      ELSE IF( INCX.EQ.0 )THEN
         INFO = 5
      ELSE IF( INCY.EQ.0 )THEN
         INFO = 7
      ELSE IF( LDA.LT.MAX( 1, M ) )THEN
         INFO = 9
      END IF
      IF( INFO.NE.0 )THEN
         CALL F06AAZ( 'F06PMF/DGER  ', INFO )
         RETURN
      END IF
C
C     Quick return if possible.
C
      IF( ( M.EQ.0 ).OR.( N.EQ.0 ).OR.( ALPHA.EQ.ZERO ) )
     $   RETURN
C
C     Start the operations. In this version the elements of A are
C     accessed sequentially with one pass through A.
C
      IF( INCY.GT.0 )THEN
         JY = 1
      ELSE
         JY = 1 - ( N - 1 )*INCY
      END IF
      IF( INCX.EQ.1 )THEN
         DO 20, J = 1, N
            IF( Y( JY ).NE.ZERO )THEN
               TEMP = ALPHA*Y( JY )
               DO 10, I = 1, M
                  A( I, J ) = A( I, J ) + X( I )*TEMP
   10          CONTINUE
            END IF
            JY = JY + INCY
   20    CONTINUE
      ELSE
         IF( INCX.GT.0 )THEN
            KX = 1
         ELSE
            KX = 1 - ( M - 1 )*INCX
         END IF
         DO 40, J = 1, N
            IF( Y( JY ).NE.ZERO )THEN
               TEMP = ALPHA*Y( JY )
               IX   = KX
               DO 30, I = 1, M
                  A( I, J ) = A( I, J ) + X( IX )*TEMP
                  IX        = IX        + INCX
   30          CONTINUE
            END IF
            JY = JY + INCY
   40    CONTINUE
      END IF
C
      RETURN
C
C     End of F06PMF (DGER  ).
C
      END
      SUBROUTINE F06PRF( UPLO, N, ALPHA, X, INCX, Y, INCY, A, LDA )
C     MARK 13 RE-ISSUE. NAG COPYRIGHT 1988.
C     .. Entry Points ..
      ENTRY      DSYR2 ( UPLO, N, ALPHA, X, INCX, Y, INCY, A, LDA )
C     .. Scalar Arguments ..
      DOUBLE PRECISION   ALPHA
      INTEGER            INCX, INCY, LDA, N
      CHARACTER*1        UPLO
C     .. Array Arguments ..
      DOUBLE PRECISION   A( LDA, * ), X( * ), Y( * )
C     ..
C
C  Purpose
C  =======
C
C  DSYR2  performs the symmetric rank 2 operation
C
C     A := alpha*x*y' + alpha*y*x' + A,
C
C  where alpha is a scalar, x and y are n element vectors and A is an n
C  by n symmetric matrix.
C
C  Parameters
C  ==========
C
C  UPLO   - CHARACTER*1.
C           On entry, UPLO specifies whether the upper or lower
C           triangular part of the array A is to be referenced as
C           follows:
C
C              UPLO = 'U' or 'u'   Only the upper triangular part of A
C                                  is to be referenced.
C
C              UPLO = 'L' or 'l'   Only the lower triangular part of A
C                                  is to be referenced.
C
C           Unchanged on exit.
C
C  N      - INTEGER.
C           On entry, N specifies the order of the matrix A.
C           N must be at least zero.
C           Unchanged on exit.
C
C  ALPHA  - DOUBLE PRECISION.
C           On entry, ALPHA specifies the scalar alpha.
C           Unchanged on exit.
C
C  X      - DOUBLE PRECISION array of dimension at least
C           ( 1 + ( n - 1 )*abs( INCX ) ).
C           Before entry, the incremented array X must contain the n
C           element vector x.
C           Unchanged on exit.
C
C  INCX   - INTEGER.
C           On entry, INCX specifies the increment for the elements of
C           X. INCX must not be zero.
C           Unchanged on exit.
C
C  Y      - DOUBLE PRECISION array of dimension at least
C           ( 1 + ( n - 1 )*abs( INCY ) ).
C           Before entry, the incremented array Y must contain the n
C           element vector y.
C           Unchanged on exit.
C
C  INCY   - INTEGER.
C           On entry, INCY specifies the increment for the elements of
C           Y. INCY must not be zero.
C           Unchanged on exit.
C
C  A      - DOUBLE PRECISION array of DIMENSION ( LDA, n ).
C           Before entry with  UPLO = 'U' or 'u', the leading n by n
C           upper triangular part of the array A must contain the upper
C           triangular part of the symmetric matrix and the strictly
C           lower triangular part of A is not referenced. On exit, the
C           upper triangular part of the array A is overwritten by the
C           upper triangular part of the updated matrix.
C           Before entry with UPLO = 'L' or 'l', the leading n by n
C           lower triangular part of the array A must contain the lower
C           triangular part of the symmetric matrix and the strictly
C           upper triangular part of A is not referenced. On exit, the
C           lower triangular part of the array A is overwritten by the
C           lower triangular part of the updated matrix.
C
C  LDA    - INTEGER.
C           On entry, LDA specifies the first dimension of A as declared
C           in the calling (sub) program. LDA must be at least
C           max( 1, n ).
C           Unchanged on exit.
C
C
C  Level 2 Blas routine.
C
C  -- Written on 22-October-1986.
C     Jack Dongarra, Argonne National Lab.
C     Jeremy Du Croz, Nag Central Office.
C     Sven Hammarling, Nag Central Office.
C     Richard Hanson, Sandia National Labs.
C
C
C     .. Parameters ..
      DOUBLE PRECISION   ZERO
      PARAMETER        ( ZERO = 0.0D+0 )
C     .. Local Scalars ..
      DOUBLE PRECISION   TEMP1, TEMP2
      INTEGER            I, INFO, IX, IY, J, JX, JY, KX, KY
C     .. External Subroutines ..
      EXTERNAL           F06AAZ
C     .. Intrinsic Functions ..
      INTRINSIC          MAX
C     ..
C     .. Executable Statements ..
C
C     Test the input parameters.
C
      INFO = 0
      IF     ( .NOT.(UPLO.EQ.'U' .OR. UPLO.EQ.'u').AND.
     $         .NOT.(UPLO.EQ.'L' .OR. UPLO.EQ.'l')      )THEN
         INFO = 1
      ELSE IF( N.LT.0 )THEN
         INFO = 2
      ELSE IF( INCX.EQ.0 )THEN
         INFO = 5
      ELSE IF( INCY.EQ.0 )THEN
         INFO = 7
      ELSE IF( LDA.LT.MAX( 1, N ) )THEN
         INFO = 9
      END IF
      IF( INFO.NE.0 )THEN
         CALL F06AAZ( 'F06PRF/DSYR2 ', INFO )
         RETURN
      END IF
C
C     Quick return if possible.
C
      IF( ( N.EQ.0 ).OR.( ALPHA.EQ.ZERO ) )
     $   RETURN
C
C     Set up the start points in X and Y if the increments are not both
C     unity.
C
      IF( ( INCX.NE.1 ).OR.( INCY.NE.1 ) )THEN
         IF( INCX.GT.0 )THEN
            KX = 1
         ELSE
            KX = 1 - ( N - 1 )*INCX
         END IF
         IF( INCY.GT.0 )THEN
            KY = 1
         ELSE
            KY = 1 - ( N - 1 )*INCY
         END IF
         JX = KX
         JY = KY
      END IF
C
C     Start the operations. In this version the elements of A are
C     accessed sequentially with one pass through the triangular part
C     of A.
C
      IF( (UPLO.EQ.'U' .OR. UPLO.EQ.'u') )THEN
C
C        Form  A  when A is stored in the upper triangle.
C
         IF( ( INCX.EQ.1 ).AND.( INCY.EQ.1 ) )THEN
            DO 20, J = 1, N
               IF( ( X( J ).NE.ZERO ).OR.( Y( J ).NE.ZERO ) )THEN
                  TEMP1 = ALPHA*Y( J )
                  TEMP2 = ALPHA*X( J )
                  DO 10, I = 1, J
                     A( I, J ) = A( I, J ) + X( I )*TEMP1 + Y( I )*TEMP2
   10             CONTINUE
               END IF
   20       CONTINUE
         ELSE
            DO 40, J = 1, N
               IF( ( X( JX ).NE.ZERO ).OR.( Y( JY ).NE.ZERO ) )THEN
                  TEMP1 = ALPHA*Y( JY )
                  TEMP2 = ALPHA*X( JX )
                  IX    = KX
                  IY    = KY
                  DO 30, I = 1, J
                     A( I, J ) = A( I, J ) + X( IX )*TEMP1
     $                                     + Y( IY )*TEMP2
                     IX        = IX        + INCX
                     IY        = IY        + INCY
   30             CONTINUE
               END IF
               JX = JX + INCX
               JY = JY + INCY
   40       CONTINUE
         END IF
      ELSE
C
C        Form  A  when A is stored in the lower triangle.
C
         IF( ( INCX.EQ.1 ).AND.( INCY.EQ.1 ) )THEN
            DO 60, J = 1, N
               IF( ( X( J ).NE.ZERO ).OR.( Y( J ).NE.ZERO ) )THEN
                  TEMP1 = ALPHA*Y( J )
                  TEMP2 = ALPHA*X( J )
                  DO 50, I = J, N
                     A( I, J ) = A( I, J ) + X( I )*TEMP1 + Y( I )*TEMP2
   50             CONTINUE
               END IF
   60       CONTINUE
         ELSE
            DO 80, J = 1, N
               IF( ( X( JX ).NE.ZERO ).OR.( Y( JY ).NE.ZERO ) )THEN
                  TEMP1 = ALPHA*Y( JY )
                  TEMP2 = ALPHA*X( JX )
                  IX    = JX
                  IY    = JY
                  DO 70, I = J, N
                     A( I, J ) = A( I, J ) + X( IX )*TEMP1
     $                                     + Y( IY )*TEMP2
                     IX        = IX        + INCX
                     IY        = IY        + INCY
   70             CONTINUE
               END IF
               JX = JX + INCX
               JY = JY + INCY
   80       CONTINUE
         END IF
      END IF
C
      RETURN
C
C     End of F06PRF (DSYR2 ).
C
      END
      SUBROUTINE F06QXF( SIDE, PIVOT, DIRECT, M, N, K1, K2, C, S, A,
     $                   LDA )
C     MARK 13 RELEASE. NAG COPYRIGHT 1988.
C     .. Scalar Arguments ..
      INTEGER            K1, K2, LDA, M, N
      CHARACTER*1        DIRECT, PIVOT, SIDE
C     .. Array Arguments ..
      DOUBLE PRECISION   A( LDA, * ), C( * ), S( * )
C     ..
C
C  F06QXF  performs the transformation
C
C     A := P*A,   when   SIDE = 'L' or 'l'  (  Left-hand side )
C
C     A := A*P',  when   SIDE = 'R' or 'r'  ( Right-hand side )
C
C  where A is an m by n matrix and P is an orthogonal matrix, consisting
C  of a  sequence  of  plane  rotations,  applied  in  planes  k1 to k2,
C  determined by the parameters PIVOT and DIRECT as follows:
C
C     When  PIVOT  = 'V' or 'v'  ( Variable pivot )
C     and   DIRECT = 'F' or 'f'  ( Forward sequence ) then
C
C        P is given as a sequence of plane rotation matrices
C
C           P = P( k2 - 1 )*...*P( k1 + 1 )*P( k1 ),
C
C        where  P( k )  is a plane rotation matrix for the  ( k, k + 1 )
C        plane.
C
C     When  PIVOT  = 'V' or 'v'  ( Variable pivot )
C     and   DIRECT = 'B' or 'b'  ( Backward sequence ) then
C
C        P is given as a sequence of plane rotation matrices
C
C           P = P( k1 )*P( k1 + 1 )*...*P( k2 - 1 ),
C
C        where  P( k )  is a plane rotation matrix for the  ( k, k + 1 )
C        plane.
C
C     When  PIVOT  = 'T' or 't'  ( Top pivot )
C     and   DIRECT = 'F' or 'f'  ( Forward sequence ) then
C
C        P is given as a sequence of plane rotation matrices
C
C           P = P( k2 - 1 )*P( k2 - 2 )*...*P( k1 ),
C
C        where  P( k )  is a plane rotation matrix for the ( k1, k + 1 )
C        plane.
C
C     When  PIVOT  = 'T' or 't'  ( Top pivot )
C     and   DIRECT = 'B' or 'b'  ( Backward sequence ) then
C
C        P is given as a sequence of plane rotation matrices
C
C           P = P( k1 )*P( k1 + 1 )*...*P( k2 - 1 ),
C
C        where  P( k )  is a plane rotation matrix for the ( k1, k + 1 )
C        plane.
C
C     When  PIVOT  = 'B' or 'b'  ( Bottom pivot )
C     and   DIRECT = 'F' or 'f'  ( Forward sequence ) then
C
C        P is given as a sequence of plane rotation matrices
C
C           P = P( k2 - 1 )*P( k2 - 2 )*...*P( k1 ),
C
C        where  P( k )  is a  plane rotation  matrix  for the  ( k, k2 )
C        plane.
C
C     When  PIVOT  = 'B' or 'b'  ( Bottom pivot )
C     and   DIRECT = 'B' or 'b'  ( Backward sequence ) then
C
C        P is given as a sequence of plane rotation matrices
C
C           P = P( k1 )*P( k1 + 1 )*...*P( k2 - 1 ),
C
C        where  P( k )  is a  plane rotation  matrix  for the  ( k, k2 )
C        plane.
C
C  c( k ) and s( k )  must contain the  cosine and sine  that define the
C  matrix  P( k ).  The  two by two  plane rotation  part of the  matrix
C  P( k ), R( k ), is assumed to be of the form
C
C     R( k ) = (  c( k )  s( k ) ).
C              ( -s( k )  c( k ) )
C
C  If m, n or k1 are less than unity,  or k2 is not greater than k1,  or
C  SIDE = 'L' or 'l'  and  k2  is greater than  m, or  SIDE = 'R' or 'r'
C  and  k2  is greater than  n,  then an  immediate return  is effected.
C
C
C  Nag Fortran 77 O( n**2 ) basic linear algebra routine.
C
C  -- Written on 20-November-1986.
C     Sven Hammarling and Mick Pont, Nag Central Office.
C
C     Alternative code used in this version when SIDE.eq.'L' to
C     enable vectorisation but working with rows of the array A.
C
C     .. Parameters ..
      DOUBLE PRECISION   ONE, ZERO
      PARAMETER          ( ONE = 1.0D+0, ZERO = 0.0D+0 )
C     .. Local Scalars ..
      DOUBLE PRECISION   CTEMP, STEMP, TEMP
      INTEGER            I, J
      LOGICAL            LEFT, RIGHT
C     .. Intrinsic Functions ..
      INTRINSIC          MIN
C     ..
C     .. Executable Statements ..
      LEFT = ( SIDE.EQ.'L' ).OR.( SIDE.EQ.'l' )
      RIGHT = ( SIDE.EQ.'R' ).OR.( SIDE.EQ.'r' )
      IF( ( MIN( M, N, K1 ).LT.1 ).OR.( K2.LE.K1 ).OR.
     $    ( ( LEFT ).AND.( K2.GT.M ) ).OR.
     $    ( ( RIGHT ).AND.( K2.GT.N ) ) )RETURN
      IF( LEFT )THEN
         IF( ( PIVOT.EQ.'V' ).OR.( PIVOT.EQ.'v' ) )THEN
            IF( ( DIRECT.EQ.'F' ).OR.( DIRECT.EQ.'f' ) )THEN
               DO 20 J = K1, K2 - 1
                  IF( ( C( J ).NE.ONE ).OR.( S( J ).NE.ZERO ) )THEN
                     CTEMP = C( J )
                     STEMP = S( J )
                     DO 10 I = 1, N
                        TEMP = A( J + 1, I )
                        A( J + 1, I ) = CTEMP*TEMP - STEMP*A( J, I )
                        A( J, I ) = STEMP*TEMP + CTEMP*A( J, I )
   10                CONTINUE
                  END IF
   20          CONTINUE
            ELSE IF( ( DIRECT.EQ.'B' ).OR.( DIRECT.EQ.'b' ) )THEN
               DO 40 J = K2 - 1, K1, -1
                  IF( ( C( J ).NE.ONE ).OR.( S( J ).NE.ZERO ) )THEN
                     CTEMP = C( J )
                     STEMP = S( J )
                     DO 30 I = 1, N
                        TEMP = A( J + 1, I )
                        A( J + 1, I ) = CTEMP*TEMP - STEMP*A( J, I )
                        A( J, I ) = STEMP*TEMP + CTEMP*A( J, I )
   30                CONTINUE
                  END IF
   40          CONTINUE
            END IF
         ELSE IF( ( PIVOT.EQ.'T' ).OR.( PIVOT.EQ.'t' ) )THEN
            IF( ( DIRECT.EQ.'F' ).OR.( DIRECT.EQ.'f' ) )THEN
               DO 60 J = K1 + 1, K2
                  CTEMP = C( J - 1 )
                  STEMP = S( J - 1 )
                  IF( ( CTEMP.NE.ONE ).OR.( STEMP.NE.ZERO ) )THEN
                     DO 50 I = 1, N
                        TEMP = A( J, I )
                        A( J, I ) = CTEMP*TEMP - STEMP*A( K1, I )
                        A( K1, I ) = STEMP*TEMP + CTEMP*A( K1, I )
   50                CONTINUE
                  END IF
   60          CONTINUE
            ELSE IF( ( DIRECT.EQ.'B' ).OR.( DIRECT.EQ.'b' ) )THEN
               DO 80 J = K2, K1 + 1, -1
                  CTEMP = C( J - 1 )
                  STEMP = S( J - 1 )
                  IF( ( CTEMP.NE.ONE ).OR.( STEMP.NE.ZERO ) )THEN
                     DO 70 I = 1, N
                        TEMP = A( J, I )
                        A( J, I ) = CTEMP*TEMP - STEMP*A( K1, I )
                        A( K1, I ) = STEMP*TEMP + CTEMP*A( K1, I )
   70                CONTINUE
                  END IF
   80          CONTINUE
            END IF
         ELSE IF( ( PIVOT.EQ.'B' ).OR.( PIVOT.EQ.'b' ) )THEN
            IF( ( DIRECT.EQ.'F' ).OR.( DIRECT.EQ.'f' ) )THEN
               DO 100 J = K1, K2 - 1
                  IF( ( C( J ).NE.ONE ).OR.( S( J ).NE.ZERO ) )THEN
                     CTEMP = C( J )
                     STEMP = S( J )
                     DO 90 I = 1, N
                        TEMP = A( J, I )
                        A( J, I ) = STEMP*A( K2, I ) + CTEMP*TEMP
                        A( K2, I ) = CTEMP*A( K2, I ) - STEMP*TEMP
   90                CONTINUE
                  END IF
  100          CONTINUE
            ELSE IF( ( DIRECT.EQ.'B' ).OR.( DIRECT.EQ.'b' ) )THEN
               DO 120 J = K2 - 1, K1, -1
                  IF( ( C( J ).NE.ONE ).OR.( S( J ).NE.ZERO ) )THEN
                     CTEMP = C( J )
                     STEMP = S( J )
                     DO 110 I = 1, N
                        TEMP = A( J, I )
                        A( J, I ) = STEMP*A( K2, I ) + CTEMP*TEMP
                        A( K2, I ) = CTEMP*A( K2, I ) - STEMP*TEMP
  110                CONTINUE
                  END IF
  120          CONTINUE
            END IF
         END IF
      ELSE IF( RIGHT )THEN
         IF( ( PIVOT.EQ.'V' ).OR.( PIVOT.EQ.'v' ) )THEN
            IF( ( DIRECT.EQ.'F' ).OR.( DIRECT.EQ.'f' ) )THEN
               DO 140 J = K1, K2 - 1
                  IF( ( C( J ).NE.ONE ).OR.( S( J ).NE.ZERO ) )THEN
                     CTEMP = C( J )
                     STEMP = S( J )
                     DO 130 I = 1, M
                        TEMP = A( I, J + 1 )
                        A( I, J + 1 ) = CTEMP*TEMP - STEMP*A( I, J )
                        A( I, J ) = STEMP*TEMP + CTEMP*A( I, J )
  130                CONTINUE
                  END IF
  140          CONTINUE
            ELSE IF( ( DIRECT.EQ.'B' ).OR.( DIRECT.EQ.'b' ) )THEN
               DO 160 J = K2 - 1, K1, -1
                  IF( ( C( J ).NE.ONE ).OR.( S( J ).NE.ZERO ) )THEN
                     CTEMP = C( J )
                     STEMP = S( J )
                     DO 150 I = 1, M
                        TEMP = A( I, J + 1 )
                        A( I, J + 1 ) = CTEMP*TEMP - STEMP*A( I, J )
                        A( I, J ) = STEMP*TEMP + CTEMP*A( I, J )
  150                CONTINUE
                  END IF
  160          CONTINUE
            END IF
         ELSE IF( ( PIVOT.EQ.'T' ).OR.( PIVOT.EQ.'t' ) )THEN
            IF( ( DIRECT.EQ.'F' ).OR.( DIRECT.EQ.'f' ) )THEN
               DO 180 J = K1 + 1, K2
                  CTEMP = C( J - 1 )
                  STEMP = S( J - 1 )
                  IF( ( CTEMP.NE.ONE ).OR.( STEMP.NE.ZERO ) )THEN
                     DO 170 I = 1, M
                        TEMP = A( I, J )
                        A( I, J ) = CTEMP*TEMP - STEMP*A( I, K1 )
                        A( I, K1 ) = STEMP*TEMP + CTEMP*A( I, K1 )
  170                CONTINUE
                  END IF
  180          CONTINUE
            ELSE IF( ( DIRECT.EQ.'B' ).OR.( DIRECT.EQ.'b' ) )THEN
               DO 200 J = K2, K1 + 1, -1
                  CTEMP = C( J - 1 )
                  STEMP = S( J - 1 )
                  IF( ( CTEMP.NE.ONE ).OR.( STEMP.NE.ZERO ) )THEN
                     DO 190 I = 1, M
                        TEMP = A( I, J )
                        A( I, J ) = CTEMP*TEMP - STEMP*A( I, K1 )
                        A( I, K1 ) = STEMP*TEMP + CTEMP*A( I, K1 )
  190                CONTINUE
                  END IF
  200          CONTINUE
            END IF
         ELSE IF( ( PIVOT.EQ.'B' ).OR.( PIVOT.EQ.'b' ) )THEN
            IF( ( DIRECT.EQ.'F' ).OR.( DIRECT.EQ.'f' ) )THEN
               DO 220 J = K1, K2 - 1
                  IF( ( C( J ).NE.ONE ).OR.( S( J ).NE.ZERO ) )THEN
                     CTEMP = C( J )
                     STEMP = S( J )
                     DO 210 I = 1, M
                        TEMP = A( I, J )
                        A( I, J ) = STEMP*A( I, K2 ) + CTEMP*TEMP
                        A( I, K2 ) = CTEMP*A( I, K2 ) - STEMP*TEMP
  210                CONTINUE
                  END IF
  220          CONTINUE
            ELSE IF( ( DIRECT.EQ.'B' ).OR.( DIRECT.EQ.'b' ) )THEN
               DO 240 J = K2 - 1, K1, -1
                  IF( ( C( J ).NE.ONE ).OR.( S( J ).NE.ZERO ) )THEN
                     CTEMP = C( J )
                     STEMP = S( J )
                     DO 230 I = 1, M
                        TEMP = A( I, J )
                        A( I, J ) = STEMP*A( I, K2 ) + CTEMP*TEMP
                        A( I, K2 ) = CTEMP*A( I, K2 ) - STEMP*TEMP
  230                CONTINUE
                  END IF
  240          CONTINUE
            END IF
         END IF
      END IF
C
      RETURN
C
C     End of F06QXF. ( SGESRC )
C
      END
      INTEGER FUNCTION P01ABF(IFAIL,IERROR,SRNAME,NREC,REC)
C     MARK 11.5(F77) RELEASE. NAG COPYRIGHT 1986.
C     MARK 13 REVISED. IER-621 (APR 1988).
C     MARK 13B REVISED. IER-668 (AUG 1988).
C
C     P01ABF is the error-handling routine for the NAG Library.
C
C     P01ABF either returns the value of IERROR through the routine
C     name (soft failure), or terminates execution of the program
C     (hard failure). Diagnostic messages may be output.
C
C     If IERROR = 0 (successful exit from the calling routine),
C     the value 0 is returned through the routine name, and no
C     message is output
C
C     If IERROR is non-zero (abnormal exit from the calling routine),
C     the action taken depends on the value of IFAIL.
C
C     IFAIL =  1: soft failure, silent exit (i.e. no messages are 
C                 output)
C     IFAIL = -1: soft failure, noisy exit (i.e. messages are output)
C     IFAIL =-13: soft failure, noisy exit but standard messages from
C                 P01ABF are suppressed
C     IFAIL =  0: hard failure, noisy exit
C
C     For compatibility with certain routines included before Mark 12
C     P01ABF also allows an alternative specification of IFAIL in which
C     it is regarded as a decimal integer with least significant digits
C     cba. Then
C
C     a = 0: hard failure  a = 1: soft failure
C     b = 0: silent exit   b = 1: noisy exit
C
C     except that hard failure now always implies a noisy exit.
C
C     S.Hammarling, M.P.Hooper and J.J.du Croz, NAG Central Office.
C
C     .. Scalar Arguments ..
      INTEGER                 IERROR, IFAIL, NREC
      CHARACTER*(*)           SRNAME
C     .. Array Arguments ..
      CHARACTER*(*)           REC(*)
C     .. Local Scalars ..
      INTEGER                 I, NERR
      CHARACTER*72            MESS
C     .. External Subroutines ..
      EXTERNAL                P01ABZ, X04AAF, X04BAF
C     .. Intrinsic Functions ..
      INTRINSIC               ABS, MOD
C     .. Executable Statements ..
      IF (IERROR.NE.0) THEN
C        Abnormal exit from calling routine
         IF (IFAIL.EQ.-1 .OR. IFAIL.EQ.0 .OR. IFAIL.EQ.-13 .OR.
     *       (IFAIL.GT.0 .AND. MOD(IFAIL/10,10).NE.0)) THEN
C           Noisy exit
            CALL X04AAF(0,NERR)
            DO 20 I = 1, NREC
               CALL X04BAF(NERR,REC(I))
   20       CONTINUE
            IF (IFAIL.NE.-13) THEN
               WRITE (MESS,FMT=99999) SRNAME, IERROR
               CALL X04BAF(NERR,MESS)
               IF (ABS(MOD(IFAIL,10)).NE.1) THEN
C                 Hard failure
                  CALL X04BAF(NERR,
     *                     ' ** NAG hard failure - execution terminated'
     *                        )
                  CALL P01ABZ
               ELSE
C                 Soft failure
                  CALL X04BAF(NERR,
     *                        ' ** NAG soft failure - control returned')
               END IF
            END IF
         END IF
      END IF
      P01ABF = IERROR
      RETURN
C
99999 FORMAT (' ** ABNORMAL EXIT from NAG Library routine ',A,': IFAIL',
     *  ' =',I6)
      END
      SUBROUTINE P01ABZ
C     MARK 11.5(F77) RELEASE. NAG COPYRIGHT 1986.
C
C     Terminates execution when a hard failure occurs.
C
C     ******************** IMPLEMENTATION NOTE ********************
C     The following STOP statement may be replaced by a call to an
C     implementation-dependent routine to display a message and/or
C     to abort the program.
C     *************************************************************
C     .. Executable Statements ..
      STOP
      END
      DOUBLE PRECISION FUNCTION X02AJF()
C     MARK 12 RELEASE. NAG COPYRIGHT 1986.
C
C     RETURNS  (1/2)*B**(1-P)  IF ROUNDS IS .TRUE.
C     RETURNS  B**(1-P)  OTHERWISE
C
      DOUBLE PRECISION X02CON
      DATA X02CON /Z'3CA0000000000001' /
C     .. Executable Statements ..
      X02AJF = X02CON
      RETURN
      END
      SUBROUTINE X04AAF(I,NERR)
C     MARK 7 RELEASE. NAG COPYRIGHT 1978
C     MARK 7C REVISED IER-190 (MAY 1979)
C     MARK 11.5(F77) REVISED. (SEPT 1985.)
C     IF I = 0, SETS NERR TO CURRENT ERROR MESSAGE UNIT NUMBER
C     (STORED IN NERR1).
C     IF I = 1, CHANGES CURRENT ERROR MESSAGE UNIT NUMBER TO
C     VALUE SPECIFIED BY NERR.
C
C     .. Scalar Arguments ..
      INTEGER           I, NERR
C     .. Local Scalars ..
      INTEGER           NERR1
C     .. Save statement ..
      SAVE              NERR1
C     .. Data statements ..
      DATA              NERR1/0/
C     .. Executable Statements ..
      IF (I.EQ.0) NERR = NERR1
      IF (I.EQ.1) NERR1 = NERR
      RETURN
      END
      SUBROUTINE X04BAF(NOUT,REC)
C     MARK 11.5(F77) RELEASE. NAG COPYRIGHT 1986.
C
C     X04BAF writes the contents of REC to the unit defined by NOUT.
C
C     Trailing blanks are not output, except that if REC is entirely
C     blank, a single blank character is output.
C     If NOUT.lt.0, i.e. if NOUT is not a valid Fortran unit identifier,
C     then no output occurs.
C
C     .. Scalar Arguments ..
      INTEGER           NOUT
      CHARACTER*(*)     REC
C     .. Local Scalars ..
      INTEGER           I
C     .. Intrinsic Functions ..
      INTRINSIC         LEN
C     .. Executable Statements ..
      IF (NOUT.GE.0) THEN
C        Remove trailing blanks
         DO 20 I = LEN(REC), 2, -1
            IF (REC(I:I).NE.' ') GO TO 40
   20    CONTINUE
C        Write record to external file
   40    WRITE (NOUT,FMT=99999) REC(1:I)
      END IF
      RETURN
C
99999 FORMAT (A)
      END
      SUBROUTINE F03AEF(N,A,IA,P,D1,ID,IFAIL)
C     MARK 14 RE-ISSUE.  NAG COPYRIGHT 1989.
C
C     CHOLDET1
C     THE UPPER TRIANGLE OF A POSITIVE DEFINITE SYMMETRIC MATRIX,
C     A, IS STORED IN THE UPPER TRIANGLE OF AN N*N ARRAY A(I,J),
C     I=1,N, J=1,N. THE CHOLESKY DECOMPOSITION A = LU, WHERE
C     U IS THE TRANSPOSE OF L, IS PERFORMED AND STORED IN THE
C     REMAINDER OF THE ARRAY A EXCEPT FOR THE RECIPROCALS OF THE
C     DIAGONAL ELEMENTS WHICH ARE STORED IN P(I), I = 1,N,
C     INSTEAD OF THE ELEMENTS THEMSELVES. A IS RETAINED SO THAT
C     THE SOLUTION OBTAINED CAN BE SUBSEQUENTLY IMPROVED. THE
C     DETERMINANT, D1 * 2.0**ID, OF A IS ALSO COMPUTED. THE
C     SUBROUTINE
C     WILL FAIL IF A, MODIFIED BY THE ROUNDING ERRORS, IS NOT
C     POSITIVE DEFINITE.
C     1ST DECEMBER 1971
C
C     .. Parameters ..
      CHARACTER*6       SRNAME
      PARAMETER         (SRNAME='F03AEF')
C     .. Scalar Arguments ..
      DOUBLE PRECISION  D1
      INTEGER           IA, ID, IFAIL, N
C     .. Array Arguments ..
      DOUBLE PRECISION  A(IA,N), P(N)
C     .. Local Scalars ..
      DOUBLE PRECISION  X, Y
      INTEGER           I, J
C     .. Local Arrays ..
      CHARACTER*1       P01REC(1)
C     .. External Functions ..
      INTEGER           P01ABF
      EXTERNAL          P01ABF
C     .. External Subroutines ..
      EXTERNAL          DGEMV
C     .. Intrinsic Functions ..
      INTRINSIC         ABS, SQRT
C     .. Executable Statements ..
      D1 = 1.0D0
      ID = 0
      DO 120 I = 1, N
         DO 20 J = I + 1, N
            A(J,I) = A(I,J)
   20    CONTINUE
         Y = A(I,I)
         CALL DGEMV('N',N-I+1,I-1,-1.0D0,A(I,1),IA,A(I,1),IA,1.0D0,
     *              A(I,I),1)
         X = A(I,I)
         A(I,I) = Y
         D1 = D1*X
         IF (X.LE.0.0D0) GO TO 140
   40    IF (ABS(D1).LT.1.0D0) GO TO 60
         D1 = D1*0.0625D0
         ID = ID + 4
         GO TO 40
   60    IF (ABS(D1).GE.0.0625D0) GO TO 80
         D1 = D1*16.0D0
         ID = ID - 4
         GO TO 60
   80    X = 1.0D0/SQRT(X)
         P(I) = X
         DO 100 J = I + 1, N
            A(J,I) = A(J,I)*X
  100    CONTINUE
  120 CONTINUE
      IFAIL = 0
      RETURN
  140 IFAIL = P01ABF(IFAIL,1,SRNAME,0,P01REC)
      RETURN
      END
      SUBROUTINE F04AGF(N,IR,A,IA,P,B,IB,X,IX)
C     MARK 2 RELEASE. NAG COPYRIGHT 1972
C     MARK 4.5 REVISED
C     MARK 11 REVISED. VECTORISATION (JAN 1984).
C     MARK 11.5(F77) REVISED. (SEPT 1985.)
C     MARK 12 REVISED. EXTENDED BLAS (JUNE 1986)
C
C     CHOLSOL1
C     SOLVES AX=B, WHERE A IS A POSITIVE DEFINITE SYMMETRIC MATRIX
C     AND B IS AN N*IR MATRIX OF IR RIGHT-HAND SIDES. THE
C     SUBROUTINE
C     F04AGF MUST BY PRECEDED BY F03AEF IN WHICH L IS
C     PRODUCED IN A(I,J) AND P(I), FROM A. AX=B IS SOLVED IN TWO
C     STEPS, LY=B AND UX=Y, AND X IS OVERWRITTEN ON Y.
C     1ST AUGUST 1971
C
C     .. Scalar Arguments ..
      INTEGER           IA, IB, IR, IX, N
C     .. Array Arguments ..
      DOUBLE PRECISION  A(IA,N), B(IB,IR), P(N), X(IX,IR)
C     .. Local Scalars ..
      DOUBLE PRECISION  T
      INTEGER           I, J
C     .. External Subroutines ..
      EXTERNAL          DTRSV
C     .. Executable Statements ..
      DO 20 I = 1, N
         T = 1.0D0/P(I)
         P(I) = A(I,I)
         A(I,I) = T
   20 CONTINUE
      DO 60 J = 1, IR
C        SOLUTION OF LY= B
         DO 40 I = 1, N
            X(I,J) = B(I,J)
   40    CONTINUE
         CALL DTRSV('L','N','N',N,A,IA,X(1,J),1)
C        SOLUTION OF UX= Y
         CALL DTRSV('L','T','N',N,A,IA,X(1,J),1)
   60 CONTINUE
      DO 80 I = 1, N
         T = P(I)
         P(I) = 1.0D0/A(I,I)
         A(I,I) = T
   80 CONTINUE
      RETURN
      END
      SUBROUTINE F02BBF(A,IA,N,ALB,UB,M,MM,R,V,IV,D,E,E2,X,G,C,ICOUNT,
     *                  IFAIL)
C     NAG COPYRIGHT 1976. MARK  5 RELEASE.
C     MARK 11.5(F77) REVISED. (SEPT 1985.)
C     MARK 13 REVISED. USE OF MARK 12 X02 FUNCTIONS (APR 1988).
C     MARK 14 REVISED. IER-740 (DEC 1989).
C     WRITTEN BY W.PHILLIPS     1ST OCTOBER 1975
C     OXFORD UNIVERSITY COMPUTING LABORATORY.
C     THIS ROUTINE REPLACES F02ACF.
C
C     SELECTED EIGENVALUES AND EIGENVECTORS OF A REAL SYMMETRIC
C     MATRIX
C
C     .. Parameters ..
      CHARACTER*6       SRNAME
      PARAMETER         (SRNAME='F02BBF')
C     .. Scalar Arguments ..
      DOUBLE PRECISION  ALB, UB
      INTEGER           IA, IFAIL, IV, M, MM, N
C     .. Array Arguments ..
      DOUBLE PRECISION  A(IA,N), D(N), E(N), E2(N), G(N), R(M), V(IV,M),
     *                  X(N,7)
      INTEGER           ICOUNT(M)
      LOGICAL           C(N)
C     .. Local Scalars ..
      DOUBLE PRECISION  EPS, XXXX, ZERO
      INTEGER           I, ISAVE
C     .. Local Arrays ..
      CHARACTER*1       P01REC(1)
C     .. External Functions ..
      DOUBLE PRECISION  X02AJF
      INTEGER           P01ABF
      EXTERNAL          X02AJF, P01ABF
C     .. External Subroutines ..
      EXTERNAL          F01AGF, F01AHF, F02BEF
C     .. Data statements ..
      DATA              ZERO/0.0D0/
C     .. Executable Statements ..
      ISAVE = IFAIL
      IFAIL = 1
      EPS = ZERO
      CALL F01AGF(N,XXXX,A,IA,D,E,E2)
      DO 20 I = 1, N
         G(I) = E(I)
   20 CONTINUE
      CALL F02BEF(N,D,ALB,UB,X02AJF(),EPS,E,E2,M,MM,R,V,IV,ICOUNT,X,C,
     *            IFAIL)
      IF (IFAIL.EQ.0) GO TO 40
      IFAIL = P01ABF(ISAVE,IFAIL,SRNAME,0,P01REC)
      RETURN
   40 IF (MM.EQ.0) RETURN
      CALL F01AHF(N,1,MM,A,IA,G,V,IV)
      RETURN
      END
      SUBROUTINE F02BEF(N,C,ALB,UB,ACHEPS,EPS,B,BETA,M,MM,ROOT,VEC,IVEC,
     *                  ICOUNT,X,LOG,IFAIL)
C     NAG COPYRIGHT 1976. MARK  5 RELEASE.
C     MARK 11.5(F77) REVISED. (SEPT 1985.)
C     WRITTEN BY W.PHILLIPS     1ST OCTOBER 1975
C     OXFORD UNIVERSITY COMPUTING LABOTATORY.
C     THIS ROUTINE REPLACES F02ASF.
C
C     TRISTURM
C     C IS THE DIAGONAL, B THE SUB-DIAGONAL AND BETA THE SQUARED
C     SUBDIAGONAL OF A SYMMETRIC TRIDIAGONAL MATRIX OF ORDER N. THE
C     EIGENVALUES WHICH ARE LESS THAN UB AND NOT LESS THAN ALB ARE
C     CALCULATED BY THE METHOD OF BISECTION AND STORED IN THE
C     VECTOR ROOT(M). THE SUBROUTINE FAILS IF M ON ENTRY IS LESS
C     THAN THE NUMBER OF EIGENVALUES REQUIRED AND ON EXIT MM GIVES
C     THE ACTUAL NUMBER OF EIGENVALUES FOUND. THE CORRESPONDING
C     EIGENVECTORS ARE CALCULATED BY INVERSE ITERATION AND ARE
C     STORED IN THE ARRAY VEC(N,M), NORMALISED SO THAT THE SUM
C     OF SQUARES IS 1, WITH THE NUMBER OF ITERATIONS STORED IN THE
C     VECTOR ICOUNT(M). THE SUBROUTINE FAILS IF ANY VECTOR HAS NOT
C     BEEN ACCEPTED AFTER 5 ITERATIONS. ELEMENTS OF B REGARDED AS
C     NEGLIGIBLE AND THE CORRESPONDING BETA ARE REPLACED BY ZERO.
C     ACHEPS IS THE RELATIVE MACHINE PRECISION AND EPS SHOULD BE
C     EQUAL TO THE ERROR TO BE TOLERATED IN THE SMALLEST
C     EIGENVALUE..
C
C     .. Parameters ..
      CHARACTER*6       SRNAME
      PARAMETER         (SRNAME='F02BEF')
C     .. Scalar Arguments ..
      DOUBLE PRECISION  ACHEPS, ALB, EPS, UB
      INTEGER           IFAIL, IVEC, M, MM, N
C     .. Array Arguments ..
      DOUBLE PRECISION  B(N), BETA(N), C(N), ROOT(M), VEC(IVEC,M),
     *                  X(N,7)
      INTEGER           ICOUNT(M)
      LOGICAL           LOG(N)
C     .. Local Scalars ..
      DOUBLE PRECISION  ANORM, BI, EPS1, EPS2, EPS3, EPS4, HALF, ONE,
     *                  THOU, TWO, U, V, X1, XM, XO, XU, ZERO
      INTEGER           I, IGROUP, II, IK, IP, IP1, IQ, IQ1, IR, IR1,
     *                  IRGROU, IS, ISAVE, ITS, J, K, M1, M2, N1
C     .. Local Arrays ..
      CHARACTER*1       P01REC(1)
C     .. External Functions ..
      INTEGER           F02BEZ, P01ABF
      EXTERNAL          F02BEZ, P01ABF
C     .. Intrinsic Functions ..
      INTRINSIC         ABS, SQRT, DBLE
C     .. Data statements ..
      DATA              ONE/1.0D0/, ZERO/0.0D0/, HALF/0.5D0/,
     *                  TWO/2.0D0/, THOU/1.0D-3/
C     .. Executable Statements ..
      EPS1 = EPS
      ISAVE = IFAIL
      IF (EPS1.LE.ZERO) EPS1 = (ABS(ALB)+ABS(UB))*ACHEPS
      IF (N.LT.2) GO TO 40
C     LOOK FOR SMALL SUB-DIAGONAL ENTRIES
      DO 20 I = 2, N
         IF (ABS(B(I)).GT.ACHEPS*(ABS(C(I))+ABS(C(I-1)))) GO TO 20
         BETA(I) = ZERO
   20 CONTINUE
   40 MM = F02BEZ(1,N,C,BETA,N,UB,ACHEPS) -
     *     F02BEZ(1,N,C,BETA,N,ALB,ACHEPS)
      IF (MM.GT.M) GO TO 1120
      IQ = 0
      IR = 1
   60 IP = IQ + 1
      N1 = N - 1
      IF (N1.LT.IP) GO TO 100
      DO 80 IQ = IP, N1
         IF (BETA(IQ+1).EQ.ZERO) GO TO 120
   80 CONTINUE
  100 IQ = N
  120 IF (IP.NE.IQ) GO TO 160
      IF (ALB.GT.C(IP) .OR. C(IP).GE.UB) GO TO 1100
      DO 140 I = 1, N
         VEC(I,IR) = ZERO
  140 CONTINUE
      ROOT(IR) = C(IP)
      VEC(IP,IR) = ONE
      ICOUNT(IR) = 0
      IR = IR + 1
      GO TO 1100
  160 M1 = F02BEZ(IP,IQ,C,BETA,N,ALB,ACHEPS) + 1
      M2 = F02BEZ(IP,IQ,C,BETA,N,UB,ACHEPS)
      IF (M1.GT.M2) GO TO 1100
C     FIND ROOTS BY BISECTION.
      XO = UB
      DO 180 I = M1, M2
         X(I,1) = UB
         X(I,2) = ALB
  180 CONTINUE
C     LOOP FOR K-TH EIGENVALUE.
      DO 320 IK = M1, M2
         K = M2 + M1 - IK
         XU = ALB
         IF (K.LT.M1) GO TO 220
         DO 200 II = M1, K
            I = M1 + K - II
            IF (XU.GE.X(I,2)) GO TO 200
            XU = X(I,2)
            GO TO 220
  200    CONTINUE
  220    IF (XO.GT.X(K,1)) XO = X(K,1)
  240    X1 = (XU+XO)*HALF
         XM = TWO*ACHEPS*(ABS(XU)+ABS(XO)) + EPS1
         IF (XM.GE.(XO-XU)) GO TO 300
         IS = F02BEZ(IP,IQ,C,BETA,N,X1,ACHEPS)
         IF (IS.GE.K) GO TO 280
         IF (IS.GE.M1) GO TO 260
         XU = X1
         X(M1,2) = X1
         GO TO 240
  260    XU = X1
         X(IS+1,2) = X1
         IF (X(IS,1).GT.X1) X(IS,1) = X1
         GO TO 240
  280    XO = X1
         GO TO 240
  300    X(K,1) = (XO+XU)*HALF
  320 CONTINUE
C     FIND VECTORS BY INVERSE ITERATION.
      ANORM = ABS(C(IP))
      IP1 = IP + 1
      IF (IQ.LT.IP1) GO TO 360
      DO 340 I = IP1, IQ
         ANORM = ANORM + ABS(C(I)) + ABS(B(I))
  340 CONTINUE
C     EPS2 IS THE CRITERION FOR GROUPING,
C     EPS3 REPLACES ZERO PIVOTS AND EQUAL ROOTS ARE
C     MODIFIED BY EPS3,
C     EPS4 IS TAKEN VERY SMALL TO AVOID OVERFLOW,
  360 EPS2 = ANORM*THOU
      EPS3 = ACHEPS*ANORM
      EPS4 = EPS3*(DBLE(IQ)-DBLE(IP)+ONE)
      IGROUP = 0
      IS = IP
      IF (M2.LT.M1) GO TO 1100
      DO 1080 K = M1, M2
         ITS = 1
         ROOT(IR) = X(K,1)
         X1 = X(K,1)
C        LOOK FOR CLOSE OR COINCIDENT ROOTS.
         IF (K.EQ.M1) GO TO 420
         IF (X1-XO.GE.EPS2) GO TO 380
         IGROUP = IGROUP + 1
         GO TO 400
  380    IGROUP = 0
  400    IF (X1.LE.XO) X1 = XO + EPS3
  420    U = EPS4/SQRT(DBLE(IQ)-DBLE(IP)+ONE)
         IF (IQ.LT.IP) GO TO 460
         DO 440 I = IP, IQ
            X(I,7) = U
  440    CONTINUE
C        ELIMINATION WITH INTERCHANGES.
  460    U = C(IP) - X1
         V = B(IP+1)
         IP1 = IP + 1
         IF (IQ.LT.IP1) GO TO 560
         DO 540 I = IP1, IQ
            J = I - 1
            BI = B(I)
            LOG(I) = ABS(BI) .GE. ABS(U)
            IF (LOG(I)) GO TO 480
            X(I,6) = BI/U
            XU = BI/U
            X(J,3) = U
            X(J,4) = V
            X(J,5) = ZERO
            U = C(I) - X1 - XU*V
            IF (I.NE.IQ) V = B(I+1)
            GO TO 540
  480       X(I,6) = U/BI
            XU = U/BI
            X(J,3) = BI
            X(J,4) = C(I) - X1
            IF (I.EQ.IQ) GO TO 500
            X(J,5) = B(I+1)
            GO TO 520
  500       X(J,5) = ZERO
  520       U = V - XU*X(J,4)
            V = -XU*X(J,5)
  540    CONTINUE
  560    X(IQ,3) = U
         IF (U.EQ.ZERO) X(IQ,3) = EPS3
         X(IQ,4) = ZERO
         X(IQ,5) = ZERO
C        BACKSUBSTITUTION.
  580    IF (IQ.LT.IP) GO TO 620
         DO 600 II = IP, IQ
            I = IP + IQ - II
            X(I,7) = (X(I,7)-U*X(I,4)-V*X(I,5))/X(I,3)
            V = U
            U = X(I,7)
  600    CONTINUE
  620    IRGROU = IR - IGROUP
         IR1 = IR - 1
         IF (IRGROU.GT.IR1) GO TO 700
         DO 680 J = IRGROU, IR1
C           ORTHOGONALISE WITH RESPECT TO PREVIOUS MEMBERS OF GROUP
            XU = ZERO
            IF (IQ.LT.IP) GO TO 680
            DO 640 I = IP, IQ
               XU = XU + X(I,7)*VEC(I,J)
  640       CONTINUE
            DO 660 I = IP, IQ
               X(I,7) = X(I,7) - XU*VEC(I,J)
  660       CONTINUE
  680    CONTINUE
  700    ANORM = ZERO
         IF (IQ.LT.IP) GO TO 740
         DO 720 I = IP, IQ
            ANORM = ANORM + ABS(X(I,7))
  720    CONTINUE
C        FORWARD SUBSTITUTION.
  740    IF (ANORM.GE.ONE) GO TO 920
         IF (ITS.NE.5) GO TO 760
         ICOUNT(IR) = 6
         GO TO 1140
  760    IF (ANORM.NE.ZERO) GO TO 800
         X(IS,7) = EPS4
         IF (IS.EQ.IQ) GO TO 780
         IS = IS + 1
         GO TO 840
  780    IS = IP
         GO TO 840
  800    XU = EPS4/ANORM
         IF (IQ.LT.IP) GO TO 840
         DO 820 I = IP, IQ
            X(I,7) = X(I,7)*XU
  820    CONTINUE
  840    IP1 = IP + 1
         IF (IQ.LT.IP1) GO TO 900
         DO 880 I = IP1, IQ
            J = I - 1
            IF (LOG(I)) GO TO 860
            X(I,7) = X(I,7) - X(I,6)*X(J,7)
            GO TO 880
  860       U = X(J,7)
            X(J,7) = X(I,7)
            X(I,7) = U - X(I,6)*X(J,7)
  880    CONTINUE
  900    ITS = ITS + 1
         GO TO 580
C        NORMALISE SO THAT SUM OF SQUARES IS 1 AND EXPAND
C        TO FULL ORDER.
  920    U = ZERO
         IF (IQ.LT.IP) GO TO 980
         DO 940 I = IP, IQ
            U = U + X(I,7)**2
  940    CONTINUE
         XU = ONE/SQRT(U)
         DO 960 I = IP, IQ
            VEC(I,IR) = X(I,7)*XU
  960    CONTINUE
  980    IP1 = IP - 1
         IF (1.GT.IP1) GO TO 1020
         DO 1000 II = 1, IP1
            I = 1 + IP1 - II
            VEC(I,IR) = ZERO
 1000    CONTINUE
 1020    IQ1 = IQ + 1
         IF (IQ1.GT.N) GO TO 1060
         DO 1040 I = IQ1, N
            VEC(I,IR) = ZERO
 1040    CONTINUE
 1060    ICOUNT(IR) = ITS
         IR = IR + 1
         XO = X1
 1080 CONTINUE
 1100 IF (IQ.LT.N) GO TO 60
      IFAIL = 0
      RETURN
 1120 IFAIL = P01ABF(ISAVE,1,SRNAME,0,P01REC)
      RETURN
 1140 IFAIL = P01ABF(ISAVE,2,SRNAME,0,P01REC)
      RETURN
      END
      SUBROUTINE F01AGF(N,ATOL,A,IA,D,E,E2)
C     MARK 14 RE-ISSUE.  NAG COPYRIGHT 1989.
C
C     TRED1
C     THIS SUBROUTINE REDUCES THE GIVEN LOWER TRIANGLE OF A
C     SYMMETRIC MATRIX, A, STORED IN THE ARRAY A(N,N), TO
C     TRIDIAGONAL FORM USING HOUSEHOLDERS REDUCTION. THE DIAGONAL
C     OF THE RESULT IS STORED IN THE ARRAY D(N) AND THE
C     SUB-DIAGONAL IN THE LAST N - 1 STORES OF THE ARRAY E(N)
C     (WITH THE ADDITIONAL ELEMENT E(1) = 0). E2(I) IS SET TO EQUAL
C     E(I)**2. THE LOWER TRIANGLE OF THE ARRAY A, TOGETHER WITH THE
C     ARRAY E, IS USED TO STORE SUFFICIENT INFORMATION FOR THE
C     DETAILS OF THE TRANSFORMATION TO BE RECOVERABLE IN THE
C     SUBROUTINE F01AHF. THE STRICTLY UPPER TRIANGLE OF THE ARRAY A
C     IS LEFT UNALTERED.
C     1ST AUGUST 1971
C
C     REVISED BY VINCE FERNANDO AT MARK 14 TO INTRODUCE SCALING INTO
C     THE GENERATION OF HOUSEHOLDER MATRICES AS PROPOSED BY
C     G.W. STEWART, INTRODUCTION TO MATRIX COMPUTATIONS, CHAPTER 7.
C     ATOL IS NOW A DUMMY PARAMETER.
C
C     .. Scalar Arguments ..
      DOUBLE PRECISION  ATOL
      INTEGER           IA, N
C     .. Array Arguments ..
      DOUBLE PRECISION  A(IA,N), D(N), E(N), E2(N)
C     .. Local Scalars ..
      DOUBLE PRECISION  F, G, H, SCALE
      INTEGER           I, II, J, K, L
C     .. External Functions ..
      DOUBLE PRECISION  DDOT
      INTEGER           IDAMAX
      EXTERNAL          DDOT, IDAMAX
C     .. External Subroutines ..
      EXTERNAL          DSCAL, DSYMV, DSYR2
C     .. Intrinsic Functions ..
      INTRINSIC         ABS, SQRT
C     .. Executable Statements ..
      DO 20 I = 1, N
         D(I) = A(N,I)
         A(N,I) = A(I,I)
   20 CONTINUE
      DO 160 II = 1, N
         I = N - II + 1
         L = I - 1
         H = 0.0D0
         IF (L.EQ.0) GO TO 60
C        FIND THE ELEMENT OF LARGEST ABSOLUTE VALUE IN D
         K = IDAMAX(L,D,1)
         SCALE = ABS(D(K))
C        IF D IS A NULL VECTOR THEN SKIP THE TRANSFORMATION
         IF (SCALE.GT.0.0D0) GO TO 80
         DO 40 J = 1, L
            D(J) = A(I-1,J)
            A(I-1,J) = A(I,J)
            A(I,J) = 0.0D0
   40    CONTINUE
   60    E(I) = 0.0D0
         E2(I) = 0.0D0
         GO TO 160
   80    CALL DSCAL(L,1.0D0/SCALE,D,1)
         H = DDOT(L,D,1,D,1)
         E2(I) = H*SCALE*SCALE
         F = D(I-1)
         G = SQRT(H)
         IF (F.GE.0.0D0) G = -G
         E(I) = G*SCALE
         H = H - F*G
         D(I-1) = F - G
C        FORM A*U
         CALL DSYMV('L',L,1.0D0/H,A,IA,D,1,0.0D0,E,1)
C        FORM P
         F = 0.0D0
         DO 100 J = 1, L
            F = F + E(J)*D(J)
  100    CONTINUE
C        FORM K
         H = F/(H+H)
C        FORM Q
         DO 120 J = 1, L
            E(J) = E(J) - H*D(J)
  120    CONTINUE
C        FORM REDUCED A
         CALL DSYR2('L',L,-1.0D0,D,1,E,1,A,IA)
         DO 140 J = 1, L
            F = D(J)
            D(J) = A(L,J)
            A(L,J) = A(I,J)
            A(I,J) = F*SCALE
  140    CONTINUE
  160 CONTINUE
      RETURN
      END
      SUBROUTINE F01AHF(N,IM1,IM2,A,IA,E,Z,IZ)
C     MARK 2 RELEASE. NAG COPYRIGHT 1972
C     MARK 4 REVISED.
C     MARK 4.5 REVISED
C     MARK 11.5(F77) REVISED. (SEPT 1985.)
C
C     TRBAK1
C     THIS SUBROUTINE PERFORMS, ON THE MATRIX OF EIGENVECTORS, Z,
C     STORED
C     IN COLUMNS IM1 TO IM2 OF THE ARRAY Z(N,N), A
C     BACKTRANSFORMATION TO
C     FORM THE EIGENVECTORS OF THE ORIGINAL SYMMETRIC MATRIX FROM
C     THE EIGENVECTORS OF THE TRIDIAGONAL MATRIX. THE NEW VECTORS
C     ARE OVERWRITTEN ON THE OLD ONES. THE SUB-DIAGONAL OF THE
C     TRIDIAGONAL MATRIX MUST BE STORED IN THE ARRAY E(N), AND
C     THE DETAILS OF THE HOUSEHOLDER REDUCTION MUST BE STORED IN
C     THE LOWER TRIANGLE OF THE ARRAY A(N,N), AS LEFT BY THE
C     SUBROUTINE F01AGF. IF Z DENOTES ANY COLUMN OF THE RESULTANT
C     MATRIX Z, THEN Z SATISFIES ZT * Z = Z(INPUT)T * Z(INPUT).
C     1ST AUGUST 1971
C
C     .. Scalar Arguments ..
      INTEGER           IA, IM1, IM2, IZ, N
C     .. Array Arguments ..
      DOUBLE PRECISION  A(IA,N), E(N), Z(IZ,IM2)
C     .. Local Scalars ..
      DOUBLE PRECISION  H, S
      INTEGER           I, J, K, L
C     .. Executable Statements ..
      IF (N.EQ.1) RETURN
      DO 80 I = 2, N
         IF (E(I).EQ.0.D0) GO TO 80
         L = I - 1
         H = E(I)*A(I,I-1)
         IF (IM1.GT.IM2) GO TO 80
         DO 60 J = IM1, IM2
            S = 0.0D0
            DO 20 K = 1, L
               S = S + A(I,K)*Z(K,J)
   20       CONTINUE
            S = S/H
            DO 40 K = 1, L
               Z(K,J) = Z(K,J) + S*A(I,K)
   40       CONTINUE
   60    CONTINUE
   80 CONTINUE
      RETURN
      END
      INTEGER FUNCTION F02BEZ(IP,IQ,D,F,N,AMBDA,ACHEPS)
C     NAG COPYRIGHT 1976. MARK  5 RELEASE.
C     MARK 11.5(F77) REVISED. (SEPT 1985.)
C     WRITTEN BY W.PHILLIPS     1ST OCTOBER 1975
C     OXFORD UNIVERSITY COMPUTING LABORATORY.
C     THIS ROUTINE REPLACES F02ASZ.
C
C     STURMCNT
C     AUXILIARY ROUTINE CALLED BY F02BEF.
C     CALCULATES THE NUMBER OF EIGENVALUES LESS THAN AMBDA
C
C     .. Scalar Arguments ..
      DOUBLE PRECISION        ACHEPS, AMBDA
      INTEGER                 IP, IQ, N
C     .. Array Arguments ..
      DOUBLE PRECISION        D(N), F(N)
C     .. Local Scalars ..
      DOUBLE PRECISION        ONE, X, ZERO
      INTEGER                 I, ICOUNT
C     .. Intrinsic Functions ..
      INTRINSIC               SQRT
C     .. Data statements ..
      DATA                    ONE/1.0D0/, ZERO/0.0D0/
C     .. Executable Statements ..
      ICOUNT = 0
      X = ONE
      DO 60 I = IP, IQ
         IF (X.EQ.ZERO) GO TO 20
         X = D(I) - AMBDA - (F(I)/X)
         GO TO 40
   20    X = D(I) - AMBDA - (SQRT(F(I)))/ACHEPS
   40    IF (X.GE.ZERO) GO TO 60
         ICOUNT = ICOUNT + 1
   60 CONTINUE
      F02BEZ = ICOUNT
      RETURN
      END
